easypackages::packages(c("tidyverse", 'sf', 'raster','sp','dplyr','spatialEco',
                         'exactextractr','tibble', 'here', "readr", "margrittr","tydr",
                         "readxl", "foreign", "data.table", "spData","spDataLarge", "purrr", "rgdal"
                         ))

remotes::install_github("Nowosad/spDataLarge")
setwd(here("Input_data"))

# Raster to point extraction  ---------------------------------------------
# Making SPAM spatial explicit 
dat_spam=read_csv("C:/Users/molch/OneDrive - Wageningen University & Research/PhD_WJS/Academic/RQ1/Data_analysis/Raw_dat/Zonation/SPAM/2010/spam2010v2r0_global_yield.csv/spam2010V2r0_global_Y_TA.csv")
dat_spam=dplyr::select(dat_spam, x,y, alloc_key, name_adm1, iso3)

# Make SPAM spatial 
# projcrs <- "+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0"
# proj_mercator = "+proj=merc +lon_0=0 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs"
proj = "+proj=longlat +datum=WGS84 +no_defs +units=m" # taken from raster file below 
proj_meter = "+datum=WGS84 +units=m +no_defs"
dat_cent <- st_as_sf(x = dat_spam,                         
                     coords = c("x", "y"),
                     crs = proj)


# dat_long = st_as_sf(coords = c("lon", "lat"))

# dat_cent = filter(dat_cent, iso3=="FIN")
# dat_cent=dat_cent[1:10,]


# import raster datasets 
dat_rast = raster("D:/Raw_dat/Raw_dat/Phosphorus/nitrogen-fertilizer-global-geotif/NFertilizer_global_geotif/nfertilizer_global.tif")
dat_rast = raster("D:/Raw_dat/Raw_dat/Phosphorus/nitrogen-fertilizer-global-geotif/NFertilizer_global_geotif/nfertilizer_global.tif")
dat_rast = raster("D:/Raw_dat/Raw_dat/Phosphorus/nitrogen-fertilizer-global-geotif/NFertilizer_global_geotif/nfertilizer_global.tif")
dat_rast_SOC = raster(here("Input_data", "sol_organic.carbon_usda.tif"))
dat_rast_peat = raster(here("Input_data", "PeatlandMap_A3_300dpi_GeoTIF_final.tif"))
dat_rast_ISCRIC = raster("D:/Raw_dat/Raw_dat/Soil maps/wise_05min_v12/wise_05min_v12/Grid/smw5by5min/w001001.adf")
dat_clim=getData('worldclim', var='bio', res=5) 
dat_rast_meantemp = dat_clim[[1]]

# Set the coordinate reference system (CRS) (i.e., define the projection).
projection(dat_rast_SOC) <- "+proj=longlat +datum=WGS84 +no_defs +units=m"

plot(clim[[1:4]])
# Import Polygons

# Point extraction of raster values and add it to the SPAM point --------
funrast = function(r){                  #insert the raster dataset from which you would like to extract values 
  r = raster::extract(r,                # raster layer
                      dat_cent,         #  SF with centroids for buffer
                      buffer = 5000,    # buffer size, units depend on CRS. Here in meters.10x10km(~5km radius)
                      fun=mean,         # what value to extract
                      df=TRUE)          # creating a df TRUE
  r$ID = dat_cent$alloc_key # assigning the identifier of the original df to the extracted df
  dat_spam <- merge(dat_spam, r, by.x = 'alloc_key', by.y = 'ID')# merge the two files together 
}



# Extraction
ex_Nfert_spam = funrast(dat_rast)
ex_soil_spam = funrast(dat_rast_ISCRIC)
ex_SOC_spam = funrast(dat_rast_SOC)
# start SOC extraction at 7.30, 19.th

filter(test, nfertilizer_global>50)
summary(test)

# Zonal statistics  -------------------------------------------------------
# Load polygons
wd=setwd("C:/Users/molch/OneDrive - Wageningen University & Research/PhD_WJS/Academic/RQ1/Data_analysis/Raw_dat/Zonation/GADM/gadm36_shp")

# Load and check shapefile 
shape <- read_sf(dsn = here("Input_data"), layer = "gadm36") # multipolygons containing all admin levels globally

# class(shape)
# shape$GID
shape=dplyr::select(shape, 'UID', 'geometry', 'GID_0', "GID_1")
# y=filter(x, GID_0 == 'ALG') 

shape_gaul=read_sf(here("Input_data", "g2008_1.shp")) #actual GAUL map 


# Load raster 
rast=dat_rast = raster("D:/Raw_dat/Raw_dat/Phosphorus/nitrogen-fertilizer-global-geotif/NFertilizer_global_geotif/nfertilizer_global.tif")


# Perform zonal statistics ------------------------------------------------
FunZone=function(rast, shape){ # raster and vector
  c=zonal.stats(shape, d, stats = c("mean")) 
  c$UID <- shape$UID  # assigning the identifier of the original df to the extracted df
  dat <- merge(shape, c, by.x = 'UID', by.y = 'UID') # merge the two files togethere
}

dat$mean.nfertilizer_global
summary(dat$mean.nfertilizer_global)



c=zonal.stats(y, dat_rast, stats = c("mean")) 
c$UID <- x$UID  # assigning the identifier of the original df to the extracted df
dat <- merge(shape, c, by.x = 'UID', by.y = 'UID') # merge the two files together 

# Point = overlay(rast, df) #does not work

BufferFun <- function(rast,ds) {
  raster::extract(rast,            # raster layer
                  ds,              #  SF with centroids for buffer
                  buffer = 5000,   # buffer size, units depend on CRS. Here in meters.10x10km(~5km radius)
                  fun=mean,        # what value to extract
                  df=TRUE)
  # ,         # creating a df TRUE
  # na.rm=TRUE)      # if is.na is true
}


cent_mean_FIN$ID = ds$alloc_key # assigning the identifier of the original df to the extracted df
centroids <- merge(dat, cent_mean_FIN, by.x = 'alloc_key', by.y = 'ID') # merge the two files together 


# Clip raster  ------------------------------------------------------------
# https://www.earthdatascience.org/courses/earth-analytics/lidar-raster-data-r/crop-raster-data-in-r/
crop_extent <- read_sf(here("Input_data", "shape_gaul_fin_adm1.shp")) 

# Plot raster 
plot(dat_rast_SOC_perc) #validated as hald of dat_rast_SOC below
# plot(dat_rast_SOC)

# Plot shapefile
plot(crop_extent %>%dplyr::select(ADM1_CODE),
     main = "Shapefile imported into R - crop extent",
     axes = T,
     border = "blue")

# crop the lidar raster using the vector extent
SOC_test_perc <- crop(dat_rast_SOC_perc, crop_extent)
plot(crop, main = "Cropped soc perc")

# add shapefile on top of the existing raster
plot(crop_extent, add = TRUE)




# Joining GADM with SPAM -------------------------------------------------------
# Load and check shapefile 
shape=read_sf(dsn = here("Input_data"), layer = "gadm36") # multipolygons containing all admin levels globally
shape=dplyr::select(shape, 'UID', 'geometry', 'GID_0', 'GID_1')
# shape_fil=filter(shape, GID_0 == 'FIN')

# Making SPAM spatial explicit 
dat_spam=read_csv(here("Input_data", "spam2010V2r0_global_Y_TA.csv"))
dat_spam=dplyr::select(dat_spam, 'x','y', 'alloc_key')

# Make SPAM spatial 
projcrs <- "+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0"
dat_spam <- st_as_sf(x = dat_spam,                         
                     coords = c("x", "y"),
                     crs = projcrs)

class(dat_cent)
# dat_cent = filter(df, iso3=="FIN")


SPAM_GADM_JOIN_GIS_left = st_join(dat_spam, shape, join = st_within, left = T) #https://postgis.net/docs/ST_Within.html 
# head(SPAM_GADM_JOIN_GIS)
# class(SPAM_GADM_Join)

st_write(obj = SPAM_GADM_JOIN_GIS, dsn = here("Input_data"), driver = "ESRI Shapefile")
st_write(obj = SPAM_GADM_JOIN_GIS_left, dsn = here("Input_data"), driver = "ESRI Shapefile")


# Joining SPAM and GAUL and GADM EU --------------------------------------------------
EU_iso3 = c("AUT","BEL", "BGR", "HRV", "CZE", "DNK", "EST","FIN", "FRA", "DEU", "GRC", "HUN", "IRL", "ITA","LVA", 
            "LTU", "LUX", "MLT", "NLD", "POL", "PRT","ROU", "SVK","SVN", "ESP", "SWE", "GBR")

EU_longGAUL = c("Austria", "Belgium","Finland", "Bulgaria", "Croatia", "Czech Republic", "Denmark", "Spain", "France",
                "Germany", "Greece", "Ireland", " Italy", " Latvia", " Lithuania", "Luxembourg","Malta", "Netherlands", 
                "Poland", "Portugal", "Rumania", "Slovakia", "Slovenia", "Estonia", "Sweden", "U.K. of Great Britain and Northern Ireland")

dat_gaul=read_sf(here("Input_data", "gaul1_asap.shp"))


dat_spam=read_csv(here("Input_data", "spam2010V2r0_global_Y_TA.csv"))
dat_spam_EU = dat_spam %>% 
  dplyr::select(alloc_key, name_adm1, name_cntr, x,y,iso3) %>% 
  filter(iso3 %in% EU_iso3)

# Make the spam dataset spatial
projcrs <- "+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0"
dat_spam_EU <- st_as_sf(x = dat_spam_EU,              
                        coords = c("x", "y"),
                        crs = projcrs)

dat_gaul=read_sf(here("Input_data", "gaul1_asap.shp"))
dat_gaul <- st_as_sf(x = dat_gaul,              
                        coords = c("x", "y"),
                        crs = projcrs)

# Source: http://worldmap.harvard.edu/data/geonode:g2008_1
dat_gaul_adm1=read_sf(here("Input_data", "g2008_1.shp"))
dat_gaul_adm1 <- st_as_sf(x = dat_gaul_adm1,              
                     coords = c("x", "y"),
                     crs = projcrs)

GADM <- read_sf(dsn = here("Input_data"), layer = "gadm36") # multipolygons containing all admin levels globally
dat_gaul <- st_as_sf(x = GADM,              
                     coords = c("x", "y"),
                     crs = projcrs)

# dat_gaul_EU = dat_gaul %>% filter(name0 %in% EU_longGAUL)

# # Joining SPAM and GAUL for EU27
# # SPAM_GAUL_JOIN_GIS = st_join(
# # ZOnal statistics > Calculating mean of SOC for EU27 
# SPAM_GAUL_JOIN_GIS_fil = SPAM_GAUL_JOIN_GIS %>% 
#   dplyr::select(alloc_key, geometry)
# top_n(n=20)

# c=zonal.stats(dat_gaul_fil, dat_rast_SOC, stats = c("mean")) 
# (dat_spam_EU, dat_gaul, join = st_within, left = T) #https://postgis.net/docs/ST_Within.html 
SPAM_GAUL_JOIN_GIS_intersect = st_join(dat_spam_EU, dat_gaul_adm1, join = st_intersects, left = T) #, left = T) 
SPAM_GAUL_GADM = st_join(SPAM_GAUL_JOIN_GIS_intersect, GADM, join = st_intersects, left = T) 

SPAM_GAUL_GADM_select = SPAM_GAUL_GADM %>% 
  dplyr::select(iso3, name_cntr, name_adm1, geometry,  #SPAM
                name1, name1_shr, name0,#                 GAUL 
                GID_1, GID_0, NAME_1, VARNAME_1) %>% #                           GADM
  write_sf(here("Input_data", "SPAM_GAUL_SF_join_intersect.shp"), driver = "ESRI Shapefile")
  
# Adding the GADM polygon geometries to the SPAM_GAUL_GADM dataset 
GADM_df = as_tibble(GADM)
SPAM_GADM_GAUL_df = as_tibble(SPAM_GAUL_GADM_select) %>% 
  dplyr::select(geometry, GID_1) %>% 
  left_join(., GADM_df, by = "GID_1")

# Extracting PEAT /non-PEAT -------------------------------------------
# Reclassify peat = 1, non=peat = 0; Peat is defined as SOC => 30% organic matter 


# create classification matrix
dat_rast_SOC_perc <- dat_rast_SOC/2 

reclass_df <- c(0, 30, 0,
                30, Inf, 1)
writeRaster(dat_rast_SOC_perc, here("Input_data", "dat_SOC_USDA_percent.tiff"))

# reshape the object into a matrix with columns and rows
reclass_m <- matrix(reclass_df,
                    ncol = 3,
                    byrow = TRUE)
# reclassify the raster using the reclass object - reclass_md
SOC_reclas <- reclassify(dat_rst_SOC_perc,
                         reclass_m)

writeRaster(SOC_reclas, here("Input_data", "SOC_reclass_perc.tif"))
SOC_reclass = raster(here("Input_data", "SOC_reclass_perc.tif"))
cellstats(SOC_reaclass)
??cellstats
cellStats(SOC_reclass, stat = max)

shape_gaul=read_sf(here("Input_data", "g2008_1.shp")) #actual GAUL map 
shape_gaul=dplyr::select(shape_gaul, 'G2008_1_ID',  'ADM1_CODE', "ADM0_CODE", 'ADM0_NAME', 'ADM1_NAME', "CONTINENT", "REGION", "geometry")

shape2 = shape_gaul %>% slice(1:1000)

extract()
?extract
EU_iso3 = c("AUT","BEL", "BGR", "HRV", "CZE", "DNK", "EST","FIN", "FRA", "DEU", "GRC", "HUN", "IRL", "ITA","LVA", 
            "LTU", "LUX", "MLT", "NLD", "POL", "PRT","ROU", "SVK","SVN", "ESP", "SWE", "GBR")
# write.csv(EU_iso3, here("Input_data", "EU27_iso3_lst.csv"))
length(EU_iso3) #27 member states > the list comes from gams

# Check if the EU countries iso3 are all present in dat.phys etc.
# EU_iso3 %in% dat.phys$iso3 #Validation if all EU27 countries are in the dfs below
shapeGL = shape_gaul %>%
  filter(ADM1_CODE == 1244) %>% 
  write_sf(here("Input_data", "shape_gaul_fin_adm1.shp"))
  


freqs = exact_extract(SOC_reclas, shapeGL, function(value, coverage_fraction)   {
    sum(coverage_fraction[value == 1], na.rm = T) / sum(coverage_fraction, na.rm = T)})

BufferFun <- function(rast,ds) {
  exact_extract(rast,            # raster layer
                as(ds, "sf"),              #  SF with centroids for buffer
                buffer = 5000)
  #,   # buffer size, units depend on CRS. Here in meters.10x10km(~5km radius)
  # fun=mean,        # what value to extract
  # df=TRUE)
  # ,         # creating a df TRUE
  # na.rm=TRUE)      # if is.na is true
}

?exact_ectract

min(SOC_reclas)
cellStats(SOC_reclas, count)


dat_spam_EU_fil = dat_spam_EU %>% 
  top_n(n=50)
proj = "+proj=longlat +datum=WGS84 +no_defs +units=m" # taken from raster file below 
proj_meter = "+datum=WGS84 +units=m +no_defs"
dat_cent <- st_as_sf(x = dat_spam_EU_fil,                         
                     coords = c("x", "y"),
                     crs = proj)
#testing the NA remove
SOC_rast_wNA = BufferFun(SOC_rast,dat_cent) 
SOC_rast_NoNa = BufferFun(dat_rast_SOC,dat_cent) #5000 m buffer, witout nA


SOC_dat_spam_EU = cbind(SOC_rast,dat_spam_EU)
write_csv(SOC_dat_spam_EU, here("Input_data", "SOC_pixel_EU.csv")) 

# Calculating percentages and Peat index
SOC_EU_pix_perc = SOC_dat_spam_EU %>% 
  mutate(percentageSOC = sol_organic.carbon_usda/2) %>% 
  mutate(peat_index = if_else(percentageSOC > 30, "Peat", "Non-peat")) %>% 
  write_csv(here("Input_data", "SOCperc_pixel_EU_Peat.csv")) 

# Then aggregate to adm1 level and bind.


cent_mean_FIN$ID = ds$alloc_key # assigning the identifier of the original df to the extracted df
centroids <- merge(dat, cent_mean_FIN, by.x = 'alloc_key', by.y = 'ID') # merge the two files together 
