easypackages::packages(c("tidyverse", 'sf', 'raster','sp', 'spatialEco',
                         'exactextractr', 'here', "readxl", "foreign", "data.table", "rbenchmark", "benchmarkme",
                         "validate", "forcats"))

# Set working directory
setwd(here::here("Input_data"))


# Functions ---------------------------------------------------------------

# This function lists all the SPAM files for the three rec_types and then reads 
# them in with a certain encoding that keeps all the names as they should be 
load_list_SPAM <- function(path, type) {
  pattern <- switch (type,
                     "harv" = c("_H_", "*.csv$"),
                     "phys" = c("_A_", "*.csv$"),
                     "prod" = c("_P_", "*.csv$"))
  path <- list.files(path = path,
                     full.names = TRUE,
                     pattern = pattern)
  l_read_csv <- lapply(path, readr::read_csv, locale = locale(encoding = "WINDOWS-1252"),#local(encoding = "WINDOWS-1252"))
                       col_types = cols(alloc_key = col_double())) #the alloc_key is set numeric 
  crop_names <- stringr::str_extract(basename(path), "^.+(?=_)")
  names(l_read_csv) <- crop_names
  return(l_read_csv)
}

# Load data  --------------------------------------------------------------

# Tabular SPAM ------------------------------------------------------------
# Production 
SPAM_prod = load_list_SPAM(path=here::here("Input_data"), type = "prod") #list of all production spam files
SPAM_prod = bind_rows(SPAM_prod)

# Harvested area
SPAM_harv = load_list_SPAM(path=here::here("Input_data"), type = "harv") #list of all harvested area spam files
SPAM_harv = bind_rows(SPAM_harv)

# Physical area
SPAM_phys = load_list_SPAM(path=here::here("Input_data"), type = "phys") #list of all physical area spam files
SPAM_phys = bind_rows(SPAM_phys)


# Spatial SPAM ------------------------------------------------------------
# Single SPAM file for spatial file creation
dat_TA_pr <- read_csv(here::here("Input_data", "spam2010V2r0_global_P_TA.csv"),locale = locale(encoding = "WINDOWS-1252"))

# Create a spatial SPAM file 
geo_spam=dplyr::select(dat_TA_pr, x,y, cell5m, name_adm1, iso3) 
# %>% 
  # distinct_at(vars(name_adm1))

# Make SPAM spatial 
# projcrs <- "+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0"
# proj_mercator = "+proj=merc +lon_0=0 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs"
proj = "+proj=longlat +datum=WGS84 +no_defs +units=m" # taken from raster file below 
proj_meter = "+datum=WGS84 +units=m +no_defs"
geo_spam1 <- st_as_sf(x = geo_spam,                         
                     coords = c("x", "y"),
                     crs = proj)

# Load administrative unit shapefile which matches the SPAM adm1 zones
# Admin0 = st_read(here::here("Input_data","GAUL_GADM_SPAM_ULRIKE", "g2008_0_now","g2008_0.shp"))
# Admin1 = st_read(here::here("Input_data","GAUL_GADM_SPAM_ULRIKE", "g2008_1_now","g2008_1.shp")) #this is what we need
# Admin2 = st_read(here::here("Input_data","GAUL_GADM_SPAM_ULRIKE", "g2008_2_2020_11_02","g2008_2.shp")) #this is what we need

# Here I manually adjusted the NAME1 values to the once used in SPAM2010. see Documentation for more. 
Admin1Adj = st_read(here::here("Input_data","g2008_1_now", "g2008_1.shp"), options = "ENCODING=WINDOWS-1252")#WINDOWS-1252

# Now the values from GGS that are still not matching the SPAM2010 adm1 values are recoded
Admin1Adj2 = Admin1Adj %>% as_tibble() %>% 
  mutate(NAME1 = case_when(ADM0_CODE == 196 & #Philipines
                            NAME1 =="Autonomous region in Muslim Mindanao (ARMM)"~ "Autonomous region in Muslim Mi",
                         TRUE ~ NAME1)) %>%
  mutate(NAME1 = case_when(ADM0_CODE == 196 & #Philipines
                           NAME1 =="Region IX (Zamboanga Peninsula)" ~ "Region IX (Zamboanga Peninsula",
                         TRUE ~ NAME1)) %>% 
  mutate(NAME1 = case_when(#ADM0_CODE == 19 & #Azerbaijan
                             NAME1 =="Administrative unit not available" ~ "Administrative unit not availa",
                           TRUE ~ NAME1)) %>% 
  mutate(NAME1 = case_when(#ADM0_CODE == 152 & #Malawi
                             NAME1 =="Area under National Administration" ~ "Area under National Administra",
                           TRUE ~ NAME1)) %>% 
  mutate(NAME1 = case_when(ADM0_CODE == 196 & #Philipines
                             NAME1 =="Cordillera Administrative region (CAR)" ~ "Cordillera Administrative regi",
                           TRUE ~ NAME1)) %>% 
  mutate(NAME1 = case_when(#ADM0_CODE == 196 & #Philipines
                             NAME1 =="Autonomous region in Muslim Mindanao (ARMM)"~ "Autonomous region in Muslim Mi",
                           TRUE ~ NAME1)) %>%
  mutate(NAME1 = case_when(ADM0_CODE == 108 & #Haiti
                             NAME1 =="NOROESTE" ~ "Nord-Est",
                           TRUE ~ NAME1)) %>% 
  mutate(NAME1 = case_when(ADM0_CODE == 111 & # Honduras
                             NAME1 =="Nor Oriental" ~ "Gracias A Dios", 
                           TRUE ~ NAME1))

# Define projection for GAULGADMSPAM
proj = "+proj=longlat +datum=WGS84 +no_defs +units=m" # taken from raster file below 
proj_meter = "+datum=WGS84 +units=m +no_defs"
AdminAdj2.2 <- st_as_sf(x = Admin1Adj2,                         
                      # coords = c("x", "y"),
                      crs = proj)

# Joining SPAM df and GAUL-GADM-SPAM --------------------------------------

# Ulrike's comment: Shape files for GAUL-GADM-SPAM
# Please note that the columns fips0, fips1 and fips2 are the ones which have the codes for admin level 0 (country), level 1 and level 2 which can be linked to SPAM results.
# The matching names are in columns Adm0_name, name1 and name2.# 
# Columns which have names O_fips1, O_fips2, O_name1 and O_name2 refer to an “old” classification system at level 1 and level 2. Its entries differ from the columns without “O_” in some EU countries and a few others where the statistic units did not match the admin units. Please note that the shape files themselves were not modified, only the entries in the dbf tables. 

# # adm1 unit match check
# spam_admin_link = dat_TA_pr%>% dplyr::select(iso3, cell5m, name_cntr, name_adm1) %>% 
#   group_by(name_adm1, iso3) %>% 
#   distinct_at(vars(name_adm1), .keep_all = T) %>% 
#   full_join(., AdminAdj2.2, by=c("name_adm1"="NAME1"))
# 
# # I need to get the adm1 units unique per country otherwise generic terms will be merged (like no "administrative units") 
# adm1_distinctAdj=AdminAdj2.2 %>% as_tibble() %>% 
#   group_by(NAME1, ADM0_NAME) %>% 
#   distinct_at(vars(NAME1), .keep_all = T)
# # str(Admin1)
# 
# # test
# test1=test %>% as_tibble() %>%
#   group_by(alpha, beta) %>% 
#   distinct_at(vars(beta), .keep_all = T)
# str(Admin1)
# 
# adm1_distinct=as_tibble(Admin1) %>% 
#   distinct_at(vars(FIPS1))#, .keep_all = T)
# 
# # adm2 units 
# adm2o_distinct=as_tibble(Admin1) %>% 
#   distinct_at(vars(O_NAME))#, .keep_all = T)
# 
# adm2_distinct=as_tibble(Admin1) %>% 
#   distinct_at(vars(FIPS1))#, .keep_all = T)
# 
# # Validation - Making sure SPAM2010 matches with admin Shapefiles 

# Adm0 - unique countries 
# SPAM2010
# 201 (name_cntr)
# 201 (iso3)

# ADM1 GAUL_SPAM_GADM
# > 275 (ADM0_NAME)
# > 250 (FIPS0)
# 

# ADM1
# 1. Same amount of rows when distinct adm1 names? 

# SPAM2010 Admin1 units
# 2493 (name_adm1)

# ADM1 GAUL_SPAM_GADM
# 3230 (NAME1)
# 3079 (O_NAME1
# 2898 (FIPS1)

# ADM2 GAUL_SPAM_GADM
#  3080 (O_NAME1)
#  2758 (NAME1)
#  3222 (O_FIPS1)
#  2898 (FIPS1)

# How to deal with inconsistency? 
# 1. Spatial join 
# 2. Fulljoin > detect NA's then check what is different

# Spatial join 
# join_SPAM_adm1 = st_join(Admin1, geo_spam1, join = st_intersects, largest = T)

# the final adm1 units are calculated by first full join and then by removing the NAs (=all variables that are not
# in the SPAM2010 and some that are not represented in the GGS (no shapes available for England subunits for example)).
# Finally, we have now 2468 adm1 units globally (of initially  2493 adm1 units; Diff: 25 adm1 units). The area of these
# 25 adm1 units needs to be added to the representative units. Otherwise, the total arable land will be off. TODO! 

# # Disinct GGS adjusted
# adm1_distinctAdj=AdminAdj2.2 %>% as_tibble() %>% 
#   group_by(NAME1, ADM0_NAME) %>% 
#   distinct_at(vars(NAME1), .keep_all = T)
# #2,741 rows
# 
# # Distinct SPAM2010
# spam_admin_link = dat_TA_pr %>% dplyr::select(iso3, cell5m, name_cntr, name_adm1) %>% 
#   group_by(name_adm1, iso3) %>%
#   distinct_at(vars(name_adm1), .keep_all = T)  
# #2,493 (diff=248)

# Left joining the SPAM2010 and GGS adjusted.
# This function's result is the final Shapefile that matches SPAM2010 adm1 
# (-17 that are not in the GGS but will be merged into other adm1 units)
FINAL_adm1_spam2010_GGS_distinct = dat_TA_pr %>% dplyr::select(iso3, cell5m, name_cntr, name_adm1) %>% 
  group_by(name_adm1, iso3) %>%
  distinct_at(vars(name_adm1), .keep_all = T) %>%
  left_join(., AdminAdj2.2, by=c("name_adm1" = "NAME1")) %>% 
  # dplyr::select(iso3, cell5m, name_adm1, ADM1_CODE, FIPS1, FIPS0) %>% 
  drop_na(ADM1_CODE) %>% 
  distinct_at(vars(name_adm1), .keep_all = T) 

# Converting to shapefile
ADM1_SPAM_GGS = st_as_sf(FINAL_adm1_spam2010_GGS_distinct)
proj = "+proj=longlat +datum=WGS84 +no_defs +units=m" # taken from raster file below 
proj_meter = "+datum=WGS84 +units=m +no_defs"
ADM1_SPAM_GGS <- st_as_sf(x = ADM1_SPAM_GGS,                         
                        # coords = c("x", "y"),
                        crs = proj)


# Writing Adm1 zone files tabular and spatial -----------------------------
write_csv(FINAL_adm1_spam2010_GGS_distinct, here::here("Input_data", "Core_data",  "ADM1_SPAM2010_GGS.csv"))
# st_write(ADM1_SPAM_GGS, here::here("Input_data", "Core_data",  "ADM1_SPAM2010_GGS_geo.shp"),
# driver = "ESRI Shapefile", options = "ENCODING=UTF-8")

# Mind the "WINDOWS-1252" encoding when reading in the ADM1_SPAM_GGS 
# ADM1_SPAM_GGS_loaded = st_read(here::here("Input_data", "Core_data",  "ADM1_SPAM2010_GGS_geo.shp"), options = "ENCODING=WINDOWS-1252") #encoding correct


# Validation questions:  --------------------------------------------------
# 1. What is the amount of adm1 units in the initial SPAM2010 data?
# SPAM2010/Adm1: Without grouping per country  = 2,493 
# SPAM2010/ADm1: With grouping per country     = 2,586  (-> Diff=93)

# 2. How many adm1 units are not represented in SPAm2010 but are in GGS? 
# with drop na: 2569 >> The amount of NAs in a left-join indicates which values in SPAM2010 are not available in GGS.
# without drop na: 5869 # Difference between so 2869-2569=300 units are not in SPAM that would actually be in GGS. 
# The number seems very high but I saw also that these reperesent many tiny islands around the world. So we just
# work with the assumption that SPAM2010 considered which adm1 units were important for crop growth and which not. 

# 3. How many units are there in the SPAM2010 but not in GGS? 
# 2586 (without drop_na) >> this indicates the df after left_join includin the na's indicating the adm1 units in SPAM but not in GGS
# 2569(with drop na) -> Difference between na and no-na is 17 -> These are the values that are in SPAM but 
# 17 adm1 units are that are in SPAM2010 are not in GGS! 

# 4. What are the adm1 units that need merging? 
# From the set of adm1 nits that are in SPAM but not in GGS we need to filter those who are combined together.
# Only then we can sum the correct area (both physical and harvested). 

# iso3	name_adm1	                  Merged with
# GBR	  North East (UK)	            East Midlands (UK)
# GBR	  North West (UK)	            East Midlands (UK)
# GBR	  Yorkshire and The Humber	  East Midlands (UK)
# GBR	  West Midlands (UK)	        East Midlands (UK)
# GBR	  East of England	            East Midlands (UK)
# GBR	  South East (UK)	            East Midlands (UK)
# GBR	  South West (UK)	            East Midlands (UK)
# GBR	  London	                    East Midlands (UK)
# CZE	  StrednÃ¬ Morava	            Strední Cechy
# MKD	  Kriva Palanka	              Kumanova
# HND	  Nor Oriental	              Gracias A Dios
# ECU	  Nororiente	                Sucumbios
# MWI	  Mzuzu	                      Karonga
# MWI	  Salima	                    Kasungu
# MWI	  Lilongwe	                  Kasungu
# MWI	  Blantyre	                  Machinga
# MWI	  Shire valley	              Machinga


# Loading treenut raster files  ------------------------------------------------
# Functions Treenuts
load_list_raster <- function(path, type) {
  pattern <- switch (type,
                     "yield" = "_YieldPerHectare\\.tif",
                     "area" = "_HarvestedAreaHectares\\.tif",
                     "prod" = "_Production\\.tif")
  path <- list.files(path = path, 
                     full.names = TRUE,
                     pattern = pattern)
  l_raster <- lapply(path, raster::raster)
  # crop_names <- stringr::str_extract(basename(path), "^.+(?=_)")
  # names(l_raster) <- crop_names
  return(l_raster)
}

# Exctracting treenuts and rest area and productio value per adm1 zones 
FunZone_sum=function(rast){ # raster and vector
  c=zonal.stats(ADM1_SPAM_GGS, rast, stats = c("sum")) 
  c$name_adm1 <- ADM1_SPAM_GGS$name_adm1  # assigning the identifier of the original df to the extracted df
  dat <- left_join(ADM1_SPAM_GGS, c, by = 'name_adm1') # merge the two files together
}

FUN_select_values = function(DF_list){
  dplyr::select(DF_list, starts_with("sum")) %>% as_tibble() %>% 
    dplyr::select(-geometry)
}

# Create factors based on other factors conditionally
FunFactCond = function(df, match_vec){
  df %>% mutate(first_tx = ifelse(name %in% match_vec, "tnuts", "rest"))
}

# Load treenut data -------------------------------------------------------
# Load harvested area and production raster data
rast_harv = load_list_raster(path=here::here("Input_data"), type = "area")
rast_prod = load_list_raster(path=here::here("Input_data"), type = "prod")

# Zonal statistics Treenuts -----------------------------------------------
lst_ex_pd = lapply(rast_prod, FunZone_sum)
lst_ex_ha = lapply(rast_harv, FunZone_sum)

# Select the extraction value columns
lst_values_pd = lapply(lst_ex_pd, FUN_select_values)
lst_values_ha = lapply(lst_ex_ha, FUN_select_values)

# Merging all lists together to a df
dat_ex_pd = do.call("cbind", lst_values_pd) # merging all tech types
dat_ex_ha = do.call("cbind", lst_values_ha)

# vector containing all tnuts names 
tnuts_vec = c("sum.walnut_Production", "sum.almond_Production","sum.chestnut_Production",
              "sum.cashew_Production","sum.pistachio_Production","sum.hazelnut_Production",
              "sum.brazil_Production","sum.nutnes_Production")


# production values tnuts adm1 --------------------------
treenut_pd = lst_ex_pd[[2]] %>% as_tibble() %>% 
  dplyr::select(iso3, cell5m , name_adm1 , G2008_1_ ,ADM0_CODE, ADM1_CODE, CONTINENT, geometry) %>% 
  bind_cols(., dat_ex_pd) %>% 
  dplyr::select(iso3, cell5m , name_adm1 , G2008_1_ ,ADM0_CODE, ADM1_CODE, 
                CONTINENT, matches("^sum\\..+_Production$"), geometry) %>%   
  group_by(iso3, name_adm1) %>% 
  mutate(treenut_pd = sum(sum.walnut_Production, 
                          sum.almond_Production,
                          sum.chestnut_Production,
                          sum.cashew_Production,
                          sum.pistachio_Production,
                          sum.hazelnut_Production,
                          sum.brazil_Production,
                          sum.nutnes_Production)) %>%
  mutate(rest_pd = sum(sum.aniseetc_Production,
                       sum.chilleetc_Production,
                       sum.kolanut_Production,
                       sum.areca_Production,
                       sum.cinnamon_Production,
                       sum.ginger_Production,
                       sum.hop_Production,
                       sum.mate_Production, 
                       sum.nutmeg_Production,
                       sum.pepper_Production,
                       sum.peppermint_Production,
                       sum.pyrethrum_Production,
                       sum.ramie_Production,
                       sum.rubber_Production,
                       sum.spicenes_Production,
                       sum.sugarnes_Production,
                       sum.vanilla_Production)) %>% 
  dplyr::select(iso3, cell5m , name_adm1 , G2008_1_ ,ADM0_CODE, ADM1_CODE, CONTINENT,treenut_pd, rest_pd)
  
  
# validation of aggregation 
# Validate_agg = xcv %>%dplyr::select(tnuts_vec, iso3, name_adm1) %>% 
  # pivot_longer(cols = starts_with("sum."),
               # names_to = "Nuts", 
               # values_to = "value")

# calcluating the harvested area values for treenuts --------------------------
treenut_ha = lst_ex_ha[[2]] %>% as_tibble() %>% 
  dplyr::select(iso3, cell5m , name_adm1 , G2008_1_ ,ADM0_CODE, ADM1_CODE, CONTINENT, geometry) %>% 
  bind_cols(., dat_ex_ha) %>% 
  dplyr::select(iso3, cell5m , name_adm1 , G2008_1_ ,ADM0_CODE, ADM1_CODE, 
                CONTINENT, matches("^sum\\..+_HarvestedAreaHectares$"), geometry) %>%   
  group_by(iso3, name_adm1) %>% 
  mutate(treenut_ha = sum(sum.walnut_HarvestedAreaHectares, 
                          sum.almond_HarvestedAreaHectares,
                          sum.chestnut_HarvestedAreaHectares,
                          sum.cashew_HarvestedAreaHectares,
                          sum.pistachio_HarvestedAreaHectares,
                          sum.hazelnut_HarvestedAreaHectares,
                          sum.brazil_HarvestedAreaHectares,
                          sum.nutnes_HarvestedAreaHectares)) %>%
  mutate(rest_ha = sum(sum.aniseetc_HarvestedAreaHectares,
                       sum.chilleetc_HarvestedAreaHectares,
                       sum.kolanut_HarvestedAreaHectares,
                       sum.areca_HarvestedAreaHectares,
                       sum.cinnamon_HarvestedAreaHectares,
                       sum.ginger_HarvestedAreaHectares,
                       sum.hop_HarvestedAreaHectares,
                       sum.mate_HarvestedAreaHectares, 
                       sum.nutmeg_HarvestedAreaHectares,
                       sum.pepper_HarvestedAreaHectares,
                       sum.peppermint_HarvestedAreaHectares,
                       sum.pyrethrum_HarvestedAreaHectares,
                       sum.ramie_HarvestedAreaHectares,
                       sum.rubber_HarvestedAreaHectares,
                       sum.spicenes_HarvestedAreaHectares,
                       sum.sugarnes_HarvestedAreaHectares,
                       sum.vanilla_HarvestedAreaHectares)) %>% 
  dplyr::select(iso3, cell5m , name_adm1 , G2008_1_ ,ADM0_CODE, ADM1_CODE, CONTINENT,treenut_ha, rest_ha)


# write and read tnuts geo data -------------------------------------------
# st_write(treenut_pd, here::here("Input_data", "Core_data",  "GSS_treenut_rest_pd.shp"),
#          driver = "ESRI Shapefile", options = "ENCODING=UTF-8")
# st_write(treenut_ha, here::here("Input_data", "Core_data",  "GSS_treenut_rest_ha.shp"),
#          driver = "ESRI Shapefile", options = "ENCODING=UTF-8")
# 
# treenut_ha = read_sf(here::here("Input_data", "GSS_treenut_rest_ha.shp"), options = "ENCODING=WINDOWS-1252")
# treenut_pd = read_sf(here::here("Input_data", "GSS_treenut_rest_pd.shp"), options = "ENCODING=WINDOWS-1252")

# # Converting to tibble
# sph.prod.tib = treenut_pd %>% as_tibble()
# sph.harv.tib = treenut_ha %>% as_tibble()

# Loading Shapefile GGS
ADM1_SPAM_GGS = st_read(here::here("Input_data", "Core_data",  "ADM1_SPAM2010_GGS_geo.shp"), options = "ENCODING=WINDOWS-1252")
ADM1_SPAM_GGS_tib = ADM1_SPAM_GGS %>% as_tibble()

# Aggregate tree nuts to to adm1 unit ---------------------------------------
# Production
prod.tib.adm1 = treenut_pd %>% as_tibble() %>%   
  dplyr::select(iso3, cell5m, name_adm1, G2008_1_ ,ADM0_CODE, ADM1_CODE, CONTINENT, treenut_pd, rest_pd) %>%
  dplyr::group_by(iso3, name_adm1, cell5m) %>% 
  dplyr::summarise(round(across(.cols = c(treenut_pd, rest_pd), .fns = sum),1))
  
  # Harvested area 
harv.tib.adm1 = treenut_ha %>%  as_tibble() %>% 
  dplyr::select(iso3, cell5m, name_adm1, G2008_1_ ,ADM0_CODE, ADM1_CODE, CONTINENT, treenut_ha, rest_ha) %>%
  dplyr::group_by(iso3, name_adm1, cell5m) %>% 
  dplyr::summarise(round(across(.cols = c(treenut_ha, rest_ha), .fns = sum),1))  #rounding is kept as in SPAM2010; 1 decimal!

# Join harvested area and calcluate tree nuts ratio  --------------------------
# Validation: Calulation of tnuts ratio correct! 
TnutsRatio_ha_pd_GGS = prod.tib.adm1 %>%
  group_by(iso3, name_adm1) %>% 
  left_join(., harv.tib.adm1, by="cell5m") %>% 
  rowwise() %>% 
  mutate(tnuts_ratio_pd = treenut_pd/(treenut_pd + rest_pd),
         tnuts_ratio_ha = treenut_ha/(treenut_ha + rest_ha)) %>%
  mutate_at(vars(tnuts_ratio_ha, tnuts_ratio_pd), ~replace(., is.nan(.), 0)) %>% 
  dplyr::rename(iso3 = iso3.x, name_adm1 = name_adm1.x) %>%
  dplyr::select(iso3, name_adm1, cell5m, tnuts_ratio_pd, tnuts_ratio_ha)

# write and read treenuts data ----------------------------------------------
write_csv(TnutsRatio_ha_pd_GGS, here::here("Input_data", "ratio_tnuts_ha_pd.csv"))
TnutsRatio_ha_pd_GGS= read_csv(here::here("Input_data", "ratio_tnuts_ha_pd.csv"))#,locale = locale(encoding = "WINDOWS-1252"))

# VEGGIES DISAGGREGATION  -------------------------------------------------
# Loading veggie raster files  ------------------------------------------------
# # Functions Veggies
# load_list_raster <- function(path, type) {
#   pattern <- switch (type,
#                      "yield" = "_YieldPerHectare\\.tif",
#                      "area" = "_HarvestedAreaHectares\\.tif",
#                      "prod" = "_Production\\.tif")
#   path <- list.files(path = path,
#                      full.names = TRUE,
#                      pattern = pattern)
#   l_raster <- lapply(path, raster::raster)
#   # crop_names <- stringr::str_extract(basename(path), "^.+(?=_)")
#   # names(l_raster) <- crop_names
#   return(l_raster)
# }
# # 
# # Exctracting veggies and rest area and production value per adm1 zones
# FunZone_sum=function(rast){ # raster and vector
#   c=zonal.stats(ADM1_SPAM_GGS, rast, stats = c("sum"))
#   c$name_adm1 <- ADM1_SPAM_GGS$name_adm1  # assigning the identifier of the original df to the extracted df
#   dat <- left_join(ADM1_SPAM_GGS, c, by = 'name_adm1') # merge the two files together
# }
# 
# FUN_select_values = function(DF_list){
#   dplyr::select(DF_list, starts_with("sum")) %>% as_tibble() %>%
#     dplyr::select(-geometry)
# }

# Create factors based on other factors conditionally
FunFactCond_veg = function(df, match_vec){
  df %>% mutate(first_tx = ifelse(name %in% match_green, "dark_green", 
                                  ifelse(name %in% match_red, "red_yellow", "other_veg")))
}

# Load treenut data -------------------------------------------------------
# Load harvested area and production raster data
rast_harv_veg = load_list_raster(path=here::here("Input_data", "Veggies_all"), type = "area")
rast_prod_veg = load_list_raster(path=here::here("Input_data", "Veggies_all"), type = "prod")

# Zonal statistics Treenuts -----------------------------------------------
lst_ex_pd_veg = lapply(rast_prod_veg, FunZone_sum)
lst_ex_ha_veg = lapply(rast_harv_veg, FunZone_sum)

# Select the extraction value columns
lst_values_pd_veg = lapply(lst_ex_pd_veg, FUN_select_values)
lst_values_ha_veg = lapply(lst_ex_ha_veg, FUN_select_values)

# Merging all lists together to a df
dat_ex_pd_veg = do.call("cbind", lst_values_pd_veg) # merging all tech types
dat_ex_ha_veg = do.call("cbind", lst_values_ha_veg)

# production values tnuts adm1 --------------------------
veggie_pd = lst_ex_pd_veg[[2]] %>% as_tibble() %>% 
  dplyr::select(iso3, cell5m , name_adm1 , G2008_1_ ,ADM0_CODE, ADM1_CODE, CONTINENT, geometry) %>% 
  bind_cols(., dat_ex_pd_veg) %>% 
  dplyr::select(iso3, cell5m , name_adm1 , G2008_1_ ,ADM0_CODE, ADM1_CODE, 
                CONTINENT, matches("^sum\\..+_Production$"), geometry) %>%   
  group_by(iso3, name_adm1) %>% 
  mutate(dark_green_pd = sum(sum.spinach_Production,   
                             sum.vetch_Production)) %>%
  mutate(red_yellow_pd = sum(sum.tomato_Production,
                             sum.pumpkinetc_Production,
                             sum.carrot_Production)) %>% 
  mutate(other_veg_pd = sum(sum.artichoke_Production,
                            sum.asparagus_Production,
                            sum.cabbage_Production,
                            sum.carob_Production,
                            sum.cauliflower_Production,
                            sum.chicory_Production,
                            sum.cucumberetc_Production,
                            sum.eggplant_Production,
                            sum.garlic_Production,
                            sum.greenbean_Production, 
                            sum.greenbroadbean_Production,
                            sum.greencorn_Production,
                            sum.greenonion_Production,
                            sum.greenpea_Production,
                            sum.legumenes_Production,
                            sum.lettuce_Production,
                            sum.mushroom_Production,
                            # sum.olive_Production,
                            sum.onion_Production,
                            sum.stringbean_Production,
                            sum.vegetablenes_Production)) %>% 
  dplyr::select(iso3, cell5m , name_adm1 , G2008_1_ ,ADM0_CODE, ADM1_CODE, CONTINENT,
                dark_green_pd, red_yellow_pd, other_veg_pd)



# validation of veggie aggregation  ---------------------------------------
# vector containing all three veggie class names 
# veg_dark_vec = c("sum.spinach_Production","sum.vetch_Production")
# veg_red_vec = c("sum.tomato_Production","sum.pumpkinetc_Production",
#                 "sum.carrot_Production")
# veg_other_vec = c("sum.artichoke_Production","sum.asparagus_Production",
#                   "sum.cabbage_Production","sum.carob_Production",
#                   "sum.cauliflower_Production","sum.chicory_Production",
#                   "sum.cucumberetc_Production","sum.eggplant_Production",
#                   "sum.garlic_Production","sum.greenbean_Production", 
#                   "sum.greenbroadbean_Production","sum.greencorn_Production",
#                   "sum.greenonion_Production","sum.greenpea_Production",
#                   "sum.legumenes_Production","sum.lettuce_Production",
#                   "sum.mushroom_Production",# sum.olive_Production,
#                   "sum.onion_Production","sum.stringbean_Production",
#                   "sum.vegetablenes_Production")
# 
# Validate_agg = veggies_pd %>%dplyr::select(tnuts_vec, iso3, name_adm1) %>% 
#   pivot_longer(cols = starts_with("sum."),
#                names_to = "veggies", 
#                values_to = "value")

# calculating the harvested area values for veggies --------------------------
veggie_ha = lst_ex_ha_veg[[2]] %>% as_tibble() %>% 
  dplyr::select(iso3, cell5m , name_adm1 , G2008_1_ ,ADM0_CODE, ADM1_CODE, CONTINENT, geometry) %>% 
  bind_cols(., dat_ex_ha_veg) %>% 
  dplyr::select(iso3, cell5m , name_adm1 , G2008_1_ ,ADM0_CODE, ADM1_CODE, 
                CONTINENT, matches("^sum\\..+_HarvestedAreaHectares$"), geometry) %>%   
  group_by(iso3, name_adm1) %>% 
  mutate(dark_green_ha = sum(sum.spinach_HarvestedAreaHectares,   
                             sum.vetch_HarvestedAreaHectares)) %>%
  mutate(red_yellow_ha = sum(sum.tomato_HarvestedAreaHectares,
                             sum.pumpkinetc_HarvestedAreaHectares,
                             sum.carrot_HarvestedAreaHectares)) %>% 
  mutate(other_veg_ha = sum(sum.artichoke_HarvestedAreaHectares,
                            sum.asparagus_HarvestedAreaHectares,
                            sum.cabbage_HarvestedAreaHectares,
                            sum.carob_HarvestedAreaHectares,
                            sum.cauliflower_HarvestedAreaHectares,
                            sum.chicory_HarvestedAreaHectares,
                            sum.cucumberetc_HarvestedAreaHectares,
                            sum.eggplant_HarvestedAreaHectares,
                            sum.garlic_HarvestedAreaHectares,
                            sum.greenbean_HarvestedAreaHectares, 
                            sum.greenbroadbean_HarvestedAreaHectares,
                            sum.greencorn_HarvestedAreaHectares,
                            sum.greenonion_HarvestedAreaHectares,
                            sum.greenpea_HarvestedAreaHectares,
                            sum.legumenes_HarvestedAreaHectares,
                            sum.lettuce_HarvestedAreaHectares,
                            sum.mushroom_HarvestedAreaHectares,
                            # sum.olive_HarvestedAreaHectares,
                            sum.onion_HarvestedAreaHectares,
                            sum.stringbean_HarvestedAreaHectares,
                            sum.vegetablenes_HarvestedAreaHectares)) %>% 
  dplyr::select(iso3, cell5m , name_adm1 , G2008_1_ ,ADM0_CODE, ADM1_CODE, CONTINENT,
                dark_green_ha, red_yellow_ha, other_veg_ha)


# write and read veggie geo data -------------------------------------------
# st_write(veggie_pd, here::here("Input_data", "Core_data",  "GSS_veggie_pd.shp"),
#          driver = "ESRI Shapefile", options = "ENCODING=UTF-8")
# st_write(veggie_ha, here::here("Input_data", "Core_data",  "GSS_veggie_ha.shp"),
#          driver = "ESRI Shapefile", options = "ENCODING=UTF-8")
# #
# veggie_ha = read_sf(here::here("Input_data", "GSS_veggie_pd.shp"), options = "ENCODING=WINDOWS-1252")
# veggie_pd = read_sf(here::here("Input_data", "GSS_veggie_ha.shp"), options = "ENCODING=WINDOWS-1252")

# # Converting to tibble
# sph.prod.tib = treenut_pd %>% as_tibble()
# sph.harv.tib = treenut_ha %>% as_tibble()


# Loading Shapefile GGS ---------------------------------------------------
# ADM1_SPAM_GGS = st_read(here::here("Input_data", "Core_data",  "ADM1_SPAM2010_GGS_geo.shp"), 
#                         options = "ENCODING=WINDOWS-1252")
# ADM1_SPAM_GGS_tib = ADM1_SPAM_GGS %>% as_tibble()

# Loading vegetable files -------------------------------------------------
# veggie_ha = read_sf(here::here("Input_data", "GSS_veggie_pd.shp"), options = "ENCODING=WINDOWS-1252")
# veggie_pd = read_sf(here::here("Input_data", "GSS_veggie_ha.shp"), options = "ENCODING=WINDOWS-1252")
  # # Converting to tibble
  # sph.prod.veg.tib = veggie_pd %>% as_tibble()
  # sph.harv.veg.tib = veggie_ha %>% as_tibble()
  
  # Loading Shapefile GGS
  ADM1_SPAM_GGS = st_read(here::here("Input_data", "Core_data",  "ADM1_SPAM2010_GGS_geo.shp"), options = "ENCODING=WINDOWS-1252")
  ADM1_SPAM_GGS_tib = ADM1_SPAM_GGS %>% as_tibble()
  
  
# Aggregate veggiesto to adm1 unit ---------------------------------------
# Production
  prod.tib.veg.adm1 = veggie_pd %>% as_tibble() %>%   
    dplyr::select(iso3, cell5m, name_adm1, G2008_1_ ,ADM0_CODE, ADM1_CODE, CONTINENT, dark_green_pd, red_yellow_pd,other_veg_pd) %>%
    dplyr::group_by(iso3, name_adm1, cell5m) %>% 
    dplyr::summarise(round(across(.cols = c(dark_green_pd, red_yellow_pd,other_veg_pd), .fns = sum),1))
  
  # Harvested area 
  harv.tib.veg.adm1 = veggie_ha %>%  as_tibble() %>% 
    dplyr::select(iso3, cell5m, name_adm1, G2008_1_ ,ADM0_CODE, ADM1_CODE, CONTINENT, dark_green_ha, red_yellow_ha,other_veg_ha) %>%
    dplyr::group_by(iso3, name_adm1, cell5m) %>% 
    dplyr::summarise(round(across(.cols = c(dark_green_ha, red_yellow_ha,other_veg_ha), .fns = sum),1))  #rounding is kept as in SPAM2010; 1 decimal!
  
  
  # Join harvested area and calcluate veggie ratio  --------------------------
  # Validation: Calulation of veggies ratio correct! adds up to 1. 
ratio_veg_ha_pd = prod.tib.veg.adm1 %>%
    group_by(iso3, name_adm1) %>% 
    left_join(., harv.tib.veg.adm1, by="cell5m") %>% 
    rowwise() %>% 
    mutate(DarkGreen_ratio_pd = dark_green_pd/(dark_green_pd+red_yellow_pd+other_veg_pd),
           DarkGreen_ratio_ha= dark_green_ha/(dark_green_ha+red_yellow_ha+other_veg_ha),
           RedYellow_ratio_pd= red_yellow_pd/(dark_green_pd+red_yellow_pd+other_veg_pd),
           RedYellow_ratio_ha= red_yellow_ha/(dark_green_ha+red_yellow_ha+other_veg_ha),
           OtherVeg_ratio_pd= other_veg_pd/(dark_green_pd+red_yellow_pd+other_veg_pd),
           OtherVeg_ratio_ha= other_veg_ha/(dark_green_ha+red_yellow_ha+other_veg_ha)) %>% 
    mutate_at(vars(DarkGreen_ratio_pd, DarkGreen_ratio_ha, 
                   RedYellow_ratio_pd, RedYellow_ratio_ha,
                   OtherVeg_ratio_pd, OtherVeg_ratio_ha), ~replace(., is.nan(.), 0)) %>% 
    rename(iso3 = iso3.x,
           name_adm1 = name_adm1.x) %>% 
    dplyr::select(cell5m, name_adm1, iso3, DarkGreen_ratio_pd, DarkGreen_ratio_ha, RedYellow_ratio_pd, 
                  RedYellow_ratio_ha, OtherVeg_ratio_pd, OtherVeg_ratio_ha)

# write and read veggie data ----------------------------------------------
write_csv(ratio_veg_ha_pd, here::here("Input_data", "ratio_veg_ha_pd.csv"))
ratio_veg_ha_pd = read_csv(here::here("Input_data", "ratio_veg_ha_pd.csv"))



# Load SPAM single SPAM files (uses less RAM for pulling through t --------


# Production
dat_TA_pr <- read_csv("spam2010V2r0_global_P_TA.csv",locale = locale(encoding = "WINDOWS-1252")) # LATIN2 encoding is used for proper display of nordic and slavic names
dat_TI_pr <- read_csv("spam2010V2r0_global_P_TI.csv",locale = locale(encoding = "WINDOWS-1252"))
dat_TH_pr <- read_csv("spam2010V2r0_global_P_TH.csv",locale = locale(encoding = "WINDOWS-1252"))
dat_TL_pr <- read_csv("spam2010V2r0_global_P_TL.csv",locale = locale(encoding = "WINDOWS-1252"))
dat_TS_pr <- read_csv("spam2010V2r0_global_P_TS.csv",locale = locale(encoding = "WINDOWS-1252"))
dat_TR_pr <- read_csv("spam2010V2r0_global_P_TR.csv",locale = locale(encoding = "WINDOWS-1252"))


# Physical area--------------------------------
dat_TA_pa <- read_csv("spam2010V2r0_global_A_TA.csv",locale = locale(encoding = "WINDOWS-1252"))
dat_TI_pa <- read_csv("spam2010V2r0_global_A_TI.csv",locale = locale(encoding = "WINDOWS-1252"))
dat_TH_pa <- read_csv("spam2010V2r0_global_A_TH.csv",locale = locale(encoding = "WINDOWS-1252"))
dat_TL_pa <- read_csv("spam2010V2r0_global_A_TL.csv",locale = locale(encoding = "WINDOWS-1252"))
dat_TS_pa <- read_csv("spam2010V2r0_global_A_TS.csv",locale = locale(encoding = "WINDOWS-1252"))
dat_TR_pa <- read_csv("spam2010V2r0_global_A_TR.csv",locale = locale(encoding = "WINDOWS-1252"))

# Harvesting area----------------------------------
dat_TA_ha <- read_csv("spam2010V2r0_global_H_TA.csv",locale = locale(encoding = "WINDOWS-1252"))
dat_TI_ha <- read_csv("spam2010V2r0_global_H_TI.csv",locale = locale(encoding = "WINDOWS-1252"))
dat_TH_ha <- read_csv("spam2010V2r0_global_H_TH.csv",locale = locale(encoding = "WINDOWS-1252"))
dat_TL_ha <- read_csv("spam2010V2r0_global_H_TL.csv",locale = locale(encoding = "WINDOWS-1252"))
dat_TS_ha <- read_csv("spam2010V2r0_global_H_TS.csv",locale = locale(encoding = "WINDOWS-1252"))
dat_TR_ha <- read_csv("spam2010V2r0_global_H_TR.csv",locale = locale(encoding = "WINDOWS-1252"))

#Aggregating SPAM to Adm1 level  -----------------------------------------------------
ChangeNames <- function(x) { ## Renaming function
  crp <- c('whea', 'rice', 'maiz', 'barl', 'pmil', 'smil', 'sorg', 'ocer', 'pota', 'swpo', 'yams', 'cass', 'orts', 'bean', 'chic', 'cowp', 'pige', 'lent', 'opul', 'soyb',
           'grou', 'cnut', 'oilp', 'sunf', 'rape', 'sesa', 'ooil', 'sugc', 'sugb', 'cott', 'ofib', 'acof', 'rcof', 'coco', 'teas', 'toba', 'bana', 'plnt', 'trof', 'temf', 'vege', 'rest')
  names(x)[10:51] <- crp
  return(x)
} 


TnutsFun_prod = function(spam){
  TnutsRatio_ha_pd_GGS %>%
    group_by(name_adm1, iso3) %>%
    right_join(., spam, by=c("iso3", "name_adm1")) %>%
    mutate(tnuts = rest*tnuts_ratio_pd, 
           rest = rest*(1-tnuts_ratio_pd)) %>%
    relocate(tnuts, .before = rest)
}

TnutsFun_area = function(spam){
  TnutsRatio_ha_pd_GGS %>%
    group_by(name_adm1, iso3) %>% 
    right_join(., spam, by=c("iso3", "name_adm1")) %>%
    mutate(tnuts = rest*tnuts_ratio_ha, 
           rest = rest*(1-tnuts_ratio_ha))%>%
    relocate(tnuts, .before = rest)
}

VeggieFun_prod = function(spam){
  ratio_veg_ha_pd %>%
    group_by(name_adm1, iso3) %>% 
    right_join(., spam, by=c("iso3", "name_adm1")) %>%
    mutate(GreenVeg = vege * DarkGreen_ratio_pd, 
           RedYelVeg = vege * RedYellow_ratio_pd, 
           RestVeg = vege*OtherVeg_ratio_pd) %>%
    relocate(GreenVeg, .before = rest) %>% 
    relocate(RedYelVeg, .before = rest) %>% 
    relocate(RestVeg, .before = rest) %>% 
    dplyr::select(-vege)
}


VeggieFun_area = function(spam){
  ratio_veg_ha_pd %>%
    group_by(name_adm1, iso3) %>% 
    right_join(., spam, by=c("iso3", "name_adm1")) %>%
    mutate(GreenVeg = vege * DarkGreen_ratio_ha, 
           RedYelVeg = vege * RedYellow_ratio_ha, 
           RestVeg = vege*OtherVeg_ratio_ha) %>%
    relocate(GreenVeg, .before = rest) %>% 
  relocate(RedYelVeg, .before = rest) %>% 
  relocate(RestVeg, .before = rest) %>% 
    dplyr::select(-vege)
}


GatherFun <- function(k){
  # k %>% pivot_longer('crop', 'valu', whea:rest)
  k %>% pivot_longer(names_to = "crop", values_to = "valu", cols = whea:rest)
}

# AggregateFun_mean <- function(z){ #for yield
#   ddply(z, .(name_adm1, crop),
#         function(z) mean(z$valu[z$valu!=0]))
# }

# AggregateFun_sum <- function(g){ #for area
#   ddply(g, .(name_adm1, crop),
#         function(g) sum(g$valu))
# }

# This function collapses adm1 units that are in SPAM but not in the GGS administrative unit file
# By combining those adm1 units, the total amount of land per country is expected to match again. 
# If we would have just left those adm1 units away, we would underestimate the area of arable land. 

Adm1MergeSPAM_GSSFUN <- function(u){
 u %>% group_by(name_adm1, iso3) %>% 
    mutate_at(c("name_adm1"), funs(recode(.,'North East (UK)'  = 'East Midlands (UK)',
                             'North West (UK)'  = 'East Midlands (UK)',
                             'Yorkshire and The Humber'  = 'East Midlands (UK)',
                             'West Midlands (UK)'  = 'East Midlands (UK)',
                             'East of England'  = 'East Midlands (UK)',
                             'South East (UK)'  = 'East Midlands (UK)',
                             'South West (UK)'  = 'East Midlands (UK)',
                             'London'  = 'East Midlands (UK)',
                             'Strední Morava' = 'Strední Cechy',
                             'Kriva Palanka'  = 'Kumanova',
                             'Nor Oriental'  = 'Gracias A Dios',
                             'Nororiente'  = 'Sucumbios',
                             'Mzuzu'  = 'Karonga',
                             'Salima'  = 'Kasungu',
                             'Lilongwe'  = 'Kasungu',
                             'Blantyre'  = 'Machinga',
                             'Shire valley'  = 'Machinga')))
}
# 
AggregateFun_sum <- function(g){ #for area
  g %>%  
  dplyr::select(iso3, name_adm1, cell5m.x, rec_type, tech_type, crop, unit, valu) %>%
  dplyr::group_by(iso3, name_adm1, crop, rec_type, tech_type, unit) %>% 
    dplyr::summarise(round(across(valu, sum),1))  #roundin is kept as in SPAM2010; 1 decimal!
}

cell5mFUN <- function(disagSPAM,aggADM1SPAM){ # getting a numeric identifier cell5m for adm1 units. 
  disagSPAM %>% dplyr::distinct_at(vars('name_adm1', 'iso3'), .keep_all = TRUE) %>% 
    dplyr::select(cell5m, iso3, name_adm1) %>% 
    group_by(iso3,name_adm1) %>% 
    right_join(.,aggADM1SPAM, by="name_adm1")
}


CleanFun <- function(l){
l %>% dplyr::select(-iso3.y) %>%  
    dplyr::rename(iso3=iso3.x) %>% 
    dplyr::select(iso3, cell5m, name_adm1, rec_type, tech_type, crop, unit, valu)
}

# Production
TransformFun_prod <- function(d){
  p <- ChangeNames(d)
  u <- Adm1MergeSPAM_GSSFUN(p)
  h <- TnutsFun_prod(u)
  o <- VeggieFun_prod(h)
  l <- GatherFun(o)
  r <- AggregateFun_sum(l)
  y <- cell5mFUN(z,r)
  j <- CleanFun(y)
}

TransformFun_area <- function(d){
p <- ChangeNames(d)
u <- Adm1MergeSPAM_GSSFUN(p)
b <- TnutsFun_area(u)
z <- VeggieFun_area(b)
l <- GatherFun(z)
r <- AggregateFun_sum(l)
y <- cell5mFUN(z,r)
q <- CleanFun(y)
}

# Applying transformation to production
TA_prd <- TransformFun_prod(dat_TA_pr)
TH_prd <- TransformFun_prod(dat_TH_pr)
TI_prd <- TransformFun_prod(dat_TI_pr)
TL_prd <- TransformFun_prod(dat_TL_pr)
TR_prd <- TransformFun_prod(dat_TR_pr)
TS_prd <- TransformFun_prod(dat_TS_pr)
lst.prd <- list(TA_prd, TH_prd, TI_prd, TL_prd, TR_prd, TS_prd)
dat_prd_prod <- do.call("rbind", lst.prd) # merging all tech types
# write_csv(dat_prd, here::here("Input_data", "Core_data", "production_all_rec_tech_adm1.csv")) #careful with encoding when writing and reading files.

#Applying transformation to physical_tech types
TA_phy <- TransformFun_prod(dat_TA_pa)
TH_phy <- TransformFun_prod(dat_TH_pa)
TI_phy <- TransformFun_prod(dat_TI_pa)
TL_phy <- TransformFun_prod(dat_TL_pa)
TR_phy <- TransformFun_prod(dat_TR_pa)
TS_phy <- TransformFun_prod(dat_TS_pa)
lst.phy <- list(TA_phy, TH_phy, TI_phy, TL_phy, TR_phy, TS_phy)
dat_phy_prod <- do.call("rbind", lst.phy) # merging all tech types
# write_csv(dat_phy, here::here("Input_data", "Core_data", "physicalarea_all_rec_tech_adm1.csv")) #careful with encoding when writing and reading files.

#Applying transformation to harvest area types
TA_har <- TransformFun_prod(dat_TA_ha)
TH_har <- TransformFun_prod(dat_TH_ha)
TI_har <- TransformFun_prod(dat_TI_ha)
TL_har <- TransformFun_prod(dat_TL_ha)
TR_har <- TransformFun_prod(dat_TR_ha)
TS_har <- TransformFun_prod(dat_TS_ha)
lst.har <- list(TA_har, TH_har, TI_har, TL_har, TR_har, TS_har)
dat_har_prod <- do.call("rbind", lst.har) # merging all tech types
# write_csv(dat_har, here::here("Input_data", "Core_data", "harvestedarea_all_rec_tech_adm1.csv"))#careful with encoding when writing and reading files.

# Validating aggregation  -------------------------------------------------
# 1. Sum up adm1 units and compare if you get same aggregation results  
# 2. Do this for all rec and tech types 
# dat_har %>%  as_tibble()
# dat_phy %>%  as_tibble()
# dat_prd %>%  as_tibble()


# Independent -- Merging treenuts with SPAM -------------------------------
# prd_dat <- read_csv(here::here("Input_data", "Core_data", "production_all_rec_tech_adm1.csv"))#, locale = locale(encoding = "LATIN2"))
# phy_dat <- read_csv(here::here("Input_data", "Core_data", "physicalarea_all_rec_tech_adm1.csv"))#, locale = locale(encoding = "LATIN2"))
# har_dat <- read_csv(here::here("Input_data", "Core_data", "harvestedarea_all_rec_tech_adm1.csv"))#, locale = locale(encoding = "LATIN2"))

# Final global transformed SPAM2010 dataset. This df is in lign with GGS adm1 units.
SPAM_GGS_all = rbind(dat_prd_area, dat_har_area, dat_phy_area)

# Changing format and calculating yields
dat_spam_global = SPAM_GGS_all %>%
  pivot_wider(!unit,
              names_from = "rec_type", 
              values_from = "valu",#,
              values_fn = mean) %>% 
  rowwise() %>% 
  mutate(Y = round((P*1000/H),1))%>% #checked for result. We are transforming mt to kg  when multiplying Production by 1000 to get unit kg/ha for yields
  mutate_at(vars(Y), ~replace(., is.nan(.), 0)) %>% 
  mutate(P = ifelse(is.infinite(Y), 0, P)) %>% #If Yields show inf values P is changed to 0.
  mutate(Y = ifelse(is.infinite(Y),0, Y)) %>%   #inf values in Y column are changed to 0's
  mutate_at(7:8, round, 1) #H and A had rounding errors. 
write.csv(dat_spam_global, here::here("Input_data", "Core_data", "SPAM_GGS_all_global_recspread_area_ratio.csv")) #Final global dataset aggregated to adm1 level and disaggregating tnuts and three veggies


# Subsetting EU -----------------------------------------------------------

# Preparing the final EU validation data based on Renee's idea ------------
# Aim is to aggregate harvested area per country

# Getting full CIFOS country names 
dat_country <- read_excel("DataValidationFinal.xlsx", 
                                  sheet = "Sheet1") %>% 
  rename(crop=crop_SPAM)
# SUBSETTING FOR EU-27 GGS  ----------------------------------------------------------------------
EU_iso3 = c("AUT","BEL", "BGR", "HRV", "CZE", "DNK", "EST","FIN", "FRA", "DEU", "GRC", "HUN", "IRL", "ITA","LVA", 
            "LTU", "LUX", "MLT", "NLD", "POL", "PRT","ROU", "SVK","SVN", "ESP", "SWE", "GBR")
# write.csv(EU_iso3, here("Input_data", "EU27_iso3_lst.csv"))
# length(EU_iso3) #27 member states > the list comes from gams

# Check if the EU countries iso3 are all present in dat.phys etc.
# EU_iso3 %in% dat.phys$iso3 #Validation if all EU27 countries are in the dfs below
dat.SPAM.EU = dat_spam_global %>% #Pixel level data
  filter(iso3 %in% EU_iso3) %>% 
  group_by(iso3, crop) %>% 
  summarise(sum(H), sum(A)) %>% 
  left_join(., dat_country, by = "iso3") %>% 
  dplyr::select(iso3, crop, `sum(H)`, `sum(A)`) %>%
  rename(HarvestedArea_ha = `sum(H)`,
         PhysicalArea_ha = `sum(A)`) #%>% 
  left_join(., dat_country, by = "crop") #%>%
  dplyr::select(iso3, Country, crop_SPAM, `sum(H)`, `sum(A)`, crop_SPAM_long, FAO_no, crop_FAO) %>% 
    


write.csv(dat.SPAM.EU, here::here("Input_data", "Core_data", "SPAM_Crops_EU27_alltecRec_nuts_veg.csv"))#, 
          # locale = locale(encoding = "LATIN2"))










# TREENUTS WITH GADM ------------------------------------------------------
# # Load and check shapefile 
# wd=setwd(here("Input_data"))
# sph.prod = read_sf(dsn = wd, layer = "GADM_treenut_rest_pd")
# sph.harv = read_sf(dsn = wd, layer = "GADM_treenut_rest_ha")
# sph.all = st_join(sph.prod, sph.harv, join = st_within)
# # write_sf(sph.all, dsn = here("Input_data", driver = "ESRI shapefile"))
# sph.prod.tib = as_tibble(sph.prod)
# sph.prod.tib = dplyr::select(sph.prod.tib, UID, GID_0, GID_1, trnt_r_)
# 
# sph.harv.tib = as_tibble(sph.harv)
# sph.harv.tib = dplyr::select(sph.harv.tib, UID, GID_0, GID_1, trnt_r_)
# 
# # loading SPAM-GADM join
# spam.gadm.link = read_sf(dsn = here("Input_data", "GADMtoSPAM", "GADM_to_SPAM_within"), 
#                          layer = "GADM_to_SPAM_within")
# # 
# # Treenuts GADM pixel  ---------------------------------------------------------------
# # CAUTION: THESE FILES ARE WRONG! THE RATIO IS WRONGLY CALULATED (not grouped by adm1 and adm0 level)
# # Treenuts are loaded from the treenuts script
# # Production
# # rbenchmark::benchmark(
# spam.gadm.link %>% 
#   as_tibble() %>% 
#   left_join(.,sph.prod.tib, by="UID") %>% 
#   left_join(.,prd_dat, by="alloc_key") %>% 
#   mutate(tnuts = trnt_r_*rest) %>% 
#   mutate(rest.1 = (1-trnt_r_)*rest) %>% 
#   dplyr::select(alloc_key, UID, GID_0.x, GID_1.x, rec_type, tech_type, unit,
#          whea:rest, tnuts, rest.1) %>% 
#   write.csv(here("Input_data", "prod_tnuts_spam_gadm.csv"))
# 
# # 
# # c=read_csv(here("Input_data", "prod_tnuts_spam_gadm.csv"))
# # 
# # Approach validation > do the numbers make sense?
# # xy=dat_join_prod %>% 
# #  group_by(tnuts) %>%
# #   top_n(n=10) %>% 
# #   dplyr::select(rest, tnuts, rest.1)
# 
# # write_csv(dat_join_prod, here("Input_data", "dat_treenut_prod.csv"))
#   # st_as_sf()
# 
# # Harvested area 
# # rbenchmark::benchmark(
# spam.gadm.link %>% 
#     as_tibble() %>% 
#     left_join(., har_dat, by="alloc_key") %>%
#     left_join(.,sph.harv.tib, by="UID") %>% 
#   mutate(tnuts = trnt_r_*rest) %>% 
#   mutate(rest.1 = (1-trnt_r_)*rest) %>% 
#   dplyr::select(alloc_key, UID, GID_0.x, GID_1.x, rec_type, tech_type, unit,
#                 whea:rest, tnuts, rest.1) %>% 
#   write.csv(here("Input_data", "harv_tnuts_spam_gadm"))
# # )
# 
# # Physical land
# # rbenchmark::benchmark(
# spam.gadm.link %>% 
#   as_tibble() %>% 
#   left_join(., phy_dat, by="alloc_key") %>%
#   left_join(.,sph.harv.tib, by="UID") %>% 
#   mutate(tnuts = trnt_r_*rest) %>% 
#   mutate(rest.1 = (1-trnt_r_)*rest) %>% 
#   dplyr::select(alloc_key, UID, GID_0.x, GID_1.x, rec_type, tech_type, unit,
#                 whea:rest, tnuts, rest.1) %>% 
#   write.csv(here("Input_data", "phys_tnuts_spam_gadm"))
# 
# # Approach: Aim is to get dinal datasets on pixel level > Adm1 level
# 1_Cbind the three datasets 
# 2_Gather crops 
# 3_Spread levels rec type
# 4_Calculate yields after unit transformation  

# # 1. Load data
# # Global 
# dat.phys = read_csv(here("Input_data", "phys_tnuts_spam_gadm.csv"))
# dat.prod = read_csv(here("Input_data", "prod_tnuts_spam_gadm,csv"))
# dat.harv = read_csv(here("Input_data", "harv_tnuts_spam_gadm.csv"))
# 
# 
# # lst.tnuts = list(dat.harv, dat.phys, dat.prod)
# dat_nutsall <- do.call("rbind", list(dat.harv, dat.phys, dat.prod))
# 
# # GLobal - Issue: Do this with the new laptop! 
# # Principle of this function is to not generate any variable as this can lead to overuse of RAM.
# do.call("rbind", list(read_csv(here("Input_data", "phys_tnuts_spam_gadm")), 
#                       read_csv(here("Input_data", "prod_tnuts_spam_gadm")), 
#                       read_csv(here("Input_data", "harv_tnuts_spam_gadm")))) %>%
#   dplyr::select(-c(rest, X1)) %>%
#   pivot_longer(names_to = "crops", values_to = "value", cols = whea:rest.1) %>% 
#   pivot_wider(-unit, names_from = "rec_type", 
#               values_from = "value" ) %>% 
#   mutate(Y = (P*1000/H))%>% #checked for result. We are transforming mt to tons when multiplying Producction by 1000 
#   mutate_at(vars(Y), ~replace(., is.nan(.), 0)) %>% 
#   rename(c("GID_0" ="GID_0.x", "GID_1"="GID_1.x")) %>% 
#   write.csv(here("Input_data", "pxl_rec_tec_gadm_spam_tnuts"))

# # SUBSETTING FOR EU-27 GADM ----------------------------------------------------------------------
# EU_iso3 = c("AUT","BEL", "BGR", "HRV", "CZE", "DNK", "EST","FIN", "FRA", "DEU", "GRC", "HUN", "IRL", "ITA","LVA", 
#             "LTU", "LUX", "MLT", "NLD", "POL", "PRT","ROU", "SVK","SVN", "ESP", "SWE", "GBR")
# write.csv(EU_iso3, here("Input_data", "EU27_iso3_lst.csv"))
# length(EU_iso3) #27 member states > the list comes from gams
# 
# # Check if the EU countries iso3 are all present in dat.phys etc.
# EU_iso3 %in% dat.phys$GID_0.x #Validation 
# dat.phys.EU = read_csv(here("Input_data", "phys_tnuts_spam_gadm"))
#   filter(dat.phys, GID_0.x %in% EU_iso3)
# 
# dat.prod.EU = read_csv(here("Input_data", "prod_tnuts_spam_gadm")) %>%
#   filter(dat.prod, GID_0.x %in% EU_iso3)
# 
# dat.harv.EU = read_csv(here("Input_data", "harv_tnuts_spam_gadm")) %>% 
# dat.harv.EU = filter(dat.harv, GID_0.x %in% EU_iso3)
# 
# 
# do.call("rbind", list(read_csv(here("Input_data", "phys_tnuts_spam_gadm")),
#                       read_csv(here("Input_data", "prod_tnuts_spam_gadm")), 
#                       read_csv(here("Input_data", "harv_tnuts_spam_gadm")))) %>% 
#   filter(GID_0.x %in% EU_iso3) %>% 
#   dplyr::select(-c(rest, X1)) %>%
#   pivot_longer(names_to = "crops", values_to = "value", cols = whea:rest.1) %>% 
#   pivot_wider(-unit, names_from = "rec_type", 
#               values_from = "value" ) %>% 
#   mutate(Y = (P*1000/H))%>% # Checked with initial spam Yield sheet (kg/ha) (original yield = 285kg/ha, here =300kg/ha) convert mt to kg Issue: Weird as according to the EU stats page aMegatonne, abbreviated as Mt, is a metric unit equivalent to 1 million (106) tonnes, or 1 billion (109) kilograms. 
#   mutate_at(vars(Y), ~replace(., is.nan(.), 0)) %>% 
#   rename(c("GID_0" ="GID_0.x", "GID_1"="GID_1.x")) %>% 
#   write.csv(here("Input_data", "pxl_rec_tec_gadm_spam_tnuts_EU27"))
# 
# # Testing the approach
# {
# # cbind dfs - testing subset
# fil.prod = dat.prod %>% top_n(n=20)
# fil.harv = dat.harv %>% top_n(n=20)
# fil.phys  = dat.phys %>% top_n(n=20)
# 
# lst.tnuts = list(fil.harv, fil.phys, fil.prod)
# do.call("rbind", list(fil.harv, fil.phys, fil.prod))
# 
# long.dat.nuts <- do.call("rbind", list(fil.harv, fil.phys, fil.prod)) %>%
#   dplyr::select(-c(X1)) %>%
#   pivot_longer(names_to = "crops", values_to = "value", cols = whea:tnuts) %>% 
#   pivot_wider(-unit, names_from = "rec_type", 
#               values_from = "value" ) %>% 
#   mutate(Y = (P*1000/H))%>% # Checked with initial spam Yield sheet (kg/ha) (original yield = 285kg/ha, here =300kg/ha) convert mt to kg Issue: Weird as according to the EU stats page aMegatonne, abbreviated as Mt, is a metric unit equivalent to 1 million (106) tonnes, or 1 billion (109) kilograms. 
#   mutate_at(vars(Y), ~replace(., is.nan(.), 0)) #%>% 
#   # rename(c("GID_0" ="GID_0.x", "GID_1"="GID_1.x")) %>% 
#   write.csv(here("Input_data", "test_rbind_fil"))
# } 



# SUBSETTING FOR EU-27 GAUL  ----------------------------------------------------------------------
EU_iso3 = c("AUT","BEL", "BGR", "HRV", "CZE", "DNK", "EST","FIN", "FRA", "DEU", "GRC", "HUN", "IRL", "ITA","LVA", 
            "LTU", "LUX", "MLT", "NLD", "POL", "PRT","ROU", "SVK","SVN", "ESP", "SWE", "GBR")
# write.csv(EU_iso3, here("Input_data", "EU27_iso3_lst.csv"))
length(EU_iso3) #27 member states > the list comes from gams

# Check if the EU countries iso3 are all present in dat.phys etc.
# EU_iso3 %in% dat.phys$iso3 #Validation if all EU27 countries are in the dfs below
dat.phys.EU = read_csv(here("Input_data", "phys_tnuts_rest_spam_gaul_FINAL.csv")) %>% #Pixel level data
  filter(iso3 %in% EU_iso3) %>% 
  mutate(alloc_key = str_sub(alloc_key, start=2)) %>% 
  dplyr::select(-c(X1_1, X1))


dat.prod.EU = read_csv(here("Input_data", "prod_tnuts_rest_spam_gaul_FINAL1.csv")) %>% 
  filter(iso3 %in% EU_iso3)%>% 
  mutate(alloc_key = str_sub(alloc_key, start=2)) %>% 
  dplyr::select(-c(X1))


dat.harv.EU = read_csv(here("Input_data", "harv_tnuts_rest_spam_gaul_FINAL.csv")) %>% 
  filter(iso3 %in% EU_iso3) %>% 
  mutate(alloc_key = str_sub(alloc_key, start=2)) %>% 
  dplyr::select(-c(X1_1, X1))


dat.all.EU=  do.call("rbind", list(dat.phys.EU, dat.prod.EU, dat.harv.EU)) %>% 
  # filter(iso3 %in% EU_iso3) %>% 
  # dplyr::select(-c(X1, X1_1)) %>%
  pivot_longer(names_to = "crops", values_to = "value", cols = whea:tnuts) %>% 
  pivot_wider(-unit, names_from = "rec_type", 
              values_from = "value" ) %>% 
  mutate(Y = (P*1000/H)) #%>% # Checked with initial spam Yield sheet (kg/ha) (original yield = 285kg/ha, here =300kg/ha) convert mt to kg Issue: Weird as according to the EU stats page aMegatonne, abbreviated as Mt, is a metric unit equivalent to 1 million (106) tonnes, or 1 billion (109) kilograms. 
mutate_at(vars(Y), ~replace(., is.nan(.), 0)) %>% 
  # rename(c("GID_0" ="GID_0.x", "GID_1"="GID_1.x")) %>% 
  write_csv(here("Input_data", "pxl_rec_tec_gaul_spam_tnuts_EU_27_corr_prod.csv")) #IMPORTANT dataset. Pixel level. Tnuts.
dat.all.EU=read_csv(here("Input_data","pxl_rec_tec_gaul_spam_tnuts_EU_27_corr_prod.csv"))

# Testing the approach
# {
# # cbind dfs - testing subset
# fil.prod = dat.prod %>% top_n(n=20)
# fil.harv = dat.harv %>% top_n(n=20)
# fil.phys  = dat.phys %>% top_n(n=20)
# 
# lst.tnuts = list(fil.harv, fil.phys, fil.prod)
# x= do.call("rbind", list(fil.harv, fil.phys, fil.prod))
# 
# long.dat.nuts <- do.call("rbind", list(fil.harv, fil.phys, fil.prod)) %>%
#   dplyr::select(-c(X1)) %>%
#   pivot_longer(names_to = "crops", values_to = "value", cols = whea:rest) %>% 
#   pivot_wider(-unit, names_from = "rec_type", 
#               values_from = "value" ) #%>% 
#   mutate(Y = (P*1000/H))%>% # Checked with initial spam Yield sheet (kg/ha) (original yield = 285kg/ha, here =300kg/ha) convert mt to kg Issue: Weird as according to the EU stats page aMegatonne, abbreviated as Mt, is a metric unit equivalent to 1 million (106) tonnes, or 1 billion (109) kilograms. 
#   mutate_at(vars(Y), ~replace(., is.nan(.), 0)) %>% 
#   rename(c("GID_0" ="GID_0.x", "GID_1"="GID_1.x")) %>% 
#   write.csv(here("Input_data", "test_rbind_fil"))
# } 

# Crop name mapping -------------------------------------------------------
# SPAM mapping 
nam_cat <- read_csv(here("Input_data", 'crop_names.csv')) # Loading crop name sheets - mapping SPAM -- FAO (FROM SPAM)
nam_cat <- nam_cat %>%  mutate(FAOCODE = as.double(FAOCODE)) %>% 
  rename(No_spam = names(nam_cat)[1])

GAEZ_SPAM_crp = read_csv(here("Input_data", "6-GAEZ-and-SPAM-crops-correspondence-2015-02-26.csv")) #Mapping GAEZ and SPAM (from SPAM)
GAEZ_SPAM_crp = GAEZ_SPAM_crp %>% 
  rename(No_spam = names(.)[4]) %>% 
  dplyr::select('SPAM crop long name', 'GAEZ crop', No_spam) 

# ORiginal Croplist Modeldat CIFOS
FAO_CIFOS_crp = read_csv(here("Input_data", "FAO_modeldat_2020.csv"))

# FAO spreaddsheets 
FAO_stat_control = read_csv(here("Input_data", "FAOSTAT_data_12-26-2020-1.csv"))
food_cat <- read_csv(here("Input_data",'foodnonfood.csv'))# laoding food usage sheet 
food_cat = food_cat %>% 
  rename(`SPAM short name`=Crop_short)
FAOcntr <- read_csv(here("Input_data", 'FAO_countries.csv')) # loading FAO country codes
FAOcntr <- dplyr::select(FAOcntr, 'ISO3', 'FAOSTAT_CODE', "GAUL_CODE", 'Continent', 'Subcontinent')


# The incomplete mapping of SPAM, GAEZ, FAOstat, FAO_Cifos are made 
# Not all the crops are matched perfectly. This is made by hand in excel (export-import a chunk below)
model_dat = left_join(FAO_CIFOS_crp, FAO_stat_control, by="crop") %>%
  left_join(nam_cat, ., by="FAOCODE") %>%
  left_join(.,GAEZ_SPAM_crp, by= "No_spam") %>% 
  write.csv(here("Input_data", "model_crops_incomplete.csv"))

# This data frame is a complete mapping of GAEZ, SPAM and FAO (FAO original and CIFOS model dat FAO crop names)
# The FAO crop names from the model data differ slightly between Model dat and the fAO crops that I exctracted from FAOSTAT. 
# Crop others were adjusted according to SPAM indication and the GAEZ mapped crops (cereal others = oats, etc.)

dat_crp_map <- read_csv("model_crops_mapping_GAEZ_SPAM_FAO_complete.csv")
# dat_crp_map2 <- read_csv(here("Input_data", "model_crops_incomplete.csv")) 

#Validation > Result = Complete match!
# dat_crp_ma p2$ crop_cifos %in% FAO_CIFOS_crp$crop 

dat_crp_map1 = dat_crp_map %>% 
  rename(crop_cifos = crop) %>% 
  left_join(.,food_cat) #adding food categories (food/non-food)

dat_crp_map1$Usage[42]="food" #adding the food category to treenuts 
write.csv(dat_crp_map1, here("Input_data", "crop_name_mapping_complete.csv")) # this file can be used for further mapping of the crops 
dat_crp_map=read_csv(here("Input_data", "crop_name_mapping_complete.csv"))# Aggregating to ADM1 level EU -----------------------------------------------


# Pxl EU27 data  ----------------------------------------------------------


dat.pxl.EU = read_csv(here("Input_data", "pxl_rec_tec_gaul_spam_tnuts_EU_27_corr_prod.csv"))
dat_TA_yld = read_csv(here("Input_data", "spam2010V2r0_global_Y_TA.csv")) %>% 
  dplyr::select(alloc_key,name_cntr,name_adm1, iso3, x, y) %>% 
  mutate(alloc_key = str_sub(alloc_key, 2))


# Mapping countries
FAOcntr <- read_csv(here("Input_data", 'FAO_countries.csv'))
FAOcntr <- dplyr::select(FAOcntr, 'ISO3', 'FAOSTAT_CODE', "GAUL_CODE", 'Continent', 'Subcontinent') %>% 
  rename(iso3=ISO3)
dat_EU27 = read_csv(here("Input_data", "Country_name_mapping_incomplete.csv"))%>% 
  left_join(.,FAOcntr, by="iso3")

dat_crop_nam = read_csv(here("Input_data", "crop_name_mapping_complete.csv")) %>% 
  dplyr::select(-c(X1,X1_1)) %>% 
  rename(crops = 'SPAM short name')

# Transform pixel data so that it has SPAM variables (adm1 level) 

dat.pixel.EU = dat.pxl.EU %>%
  mutate_at(vars(alloc_key), funs(as.character)) %>%  
  left_join(.,dat_TA_yld, by="alloc_key") %>%
  dplyr::select(alloc_key, iso3.x, name_cntr.x,name_adm1.x, iso3.x, G2008_1_ID, 
                ADM0_CODE, ADM0_NAME, ADM1_CODE, ADM1_NAME, x, y, crops, tech_type) %>% 
  rename(iso3=iso3.x,
         name_cntr=name_cntr.x,
         name_adm1=name_adm1.x) %>% 
  left_join(.,dat_crop_nam, by="crops") %>% 
  left_join(.,dat_EU27, by="iso3") %>% 
  dplyr::select(G2008_1_ID, tech_type, CIFOS_cntr_nam, crop_cifos, crops) 

# df with prod_level and Cifos country EU
EU_iso3 = c("AUT","BEL", "BGR", "HRV", "CZE", "DNK", "EST","FIN", "FRA", "DEU", "GRC", "HUN", "IRL", "ITA","LVA", 
            "LTU", "LUX", "MLT", "NLD", "POL", "PRT","ROU", "SVK","SVN", "ESP", "SWE", "GBR")

dat_cntr_EU = dat.pxl.EU %>%
  mutate_at(vars(alloc_key), funs(as.character)) %>%
  left_join(.,dat_TA_yld, by="alloc_key") %>%
  dplyr::select(iso3.x, name_adm1.x) %>% 
  dplyr::distinct_at(vars('iso3.x', "name_adm1.x"), .keep_all = F) %>%
  rename(iso3=iso3.x, 
         name_adm1=name_adm1.x) %>% 
  filter(iso3 %in% EU_iso3) %>% 
  left_join(.,dat_EU27, by="iso3") %>% 
  dplyr::select(iso3, name_adm1, CIFOS_cntr_nam, Continent, Subcontinent)# %>% 
dat_cntr_EU %>%  write_csv(here("Input_data", "dat_cntr_EU.csv"))

# rename(
# iso3=iso3.x, 
# name_adm1=name_adm1.x)

# df with crops and cifos crops
dat_crop_nam1 = read_csv(here("Input_data", "crop_name_mapping_complete.csv")) %>% 
  dplyr::select('SPAM short name', FAONAMES, crop_cifos) %>% 
  rename(crops = 'SPAM short name')

# Aggregating H,A,P and calculating Yields per Adm1 level 
dat.adm1.EU = dat.pxl.EU %>%
  mutate_at(vars(alloc_key), funs(as.character)) %>%
  #   left_join(., dat.pxl.EU, by="ADM1_NAME") %>% 
  # mutate_at(vars(alloc_key), funs(as.character)) %>%
  left_join(.,dat_TA_yld, by="alloc_key") %>%
  dplyr::select(iso3.x, name_cntr.x,  name_adm1.x, ADM0_NAME, ADM1_NAME, tech_type, crops, A, H, P) %>%
  group_by(iso3.x, name_adm1.x, tech_type, crops) %>% 
  summarize(across(.cols = A:P, .fns = sum))%>% #all three rec types are summed per adm1(prod_level) and crop
  mutate(Y = (P*1000/H))%>% # Checked with initial spam Yield sheet (kg/ha) (original yield = 285kg/ha, here =300kg/ha) convert mt to kg Issue: Weird as according to the EU stats page aMegatonne, abbreviated as Mt, is a metric unit equivalent to 1 million (106) tonnes, or 1 billion (109) kilograms. 
  mutate_at(vars(Y), ~replace(., is.nan(.), 0)) #%>% 

# This is the sheet that contains the parameters that are important for the GAMS input file
# It contains yield, harvested and physical area per country, adm1 unit and crop.
EU27_crp_adm1 = dat.adm1.EU %>% 
  left_join(.,dat_cntr_EU, by=c("name_adm1.x"="name_adm1")) %>% 
  filter(iso3 %in% EU_iso3) %>%
  left_join(.,dat_crop_nam1, by="crops") %>% 
  dplyr::select(iso3, FAONAMES, Subcontinent, Continent, CIFOS_cntr_nam, name_adm1, crop_cifos, A, H, P, Y) %>% 
  write.csv(here("Input_data", "EU27_crp_adm1.csv")) # Final crop yield,area,etc. file. 

# Validate
EU27_crp_adm1 <- read_csv("C:/Users/simon083/OneDrive - Wageningen University & Research/PhD_WJS/Academic/RQ1/Data_analysis/Spatial_mod/Modified_scripts/R/CifoS_Crop/Input_data/EU27_crp_adm1.csv")
View(EU27_crp_adm1)

length(unique(EU27_crp_adm1$iso3))
is.na(EU27_crp_adm1)#Correct! 27 countries present. 
# Continue: Why is ALB in the final dataset. Check where this went wrong 
# Validate Yields. Sum the ADM1 by hand. Check if matches 
# Berechne ob das sinn macht > 319652 rows ?!


# Then add again the GADM and get values for the SOC% >> ADd the peat non peat to it. 
# ADd usage to the sheet (or single sheet)
#

# Bringing back some SPAM geo-variables to compare them with GADM 
dat.agg.EU = dat_TA_yld %>% 
  dplyr::select(iso3, alloc_key, x, y,name_cntr, name_adm1) %>% 
  right_join(.,dat.pxl.EU, by="alloc_key")




dat.agg.EU %>% dplyr::select(iso3) dat.pxl.EU %>% 
  # dplyr::select(-Y) #%>% 
  group_by(GID_1) %>% 
  summarize

max(dat.pxl.EU$Y)
summarise()

v <- validator()
summary(v)
cf <- confront(dat.pxl.EU, v)
summary(cf)
?confront



# creating a tidy table with all tech variables in one df
list.comb <- list(yld_dat, prod_dat, har_dat, phy_dat)
list.comb2=list(dat_har, dat_phy, dat_yld, da)
dat.comb <- do.call("rbind", list.comb)
# write.csv(dat.comb, 'C:/Users/molch/OneDrive - Wageningen University & Research/PhD_WJS/Academic/RQ1/Data_analysis/Spatial_mod/Output_dat/dat_comb_tidy.csv', row.names=T)





# For ArcGIS - area physical per pixel  --------------------------

setwd(here("Input_data"))
# Physical area--------------------------------
dat_TA_pa <- read_csv("spam2010V2r0_global_A_TA.csv")
dat_TA_pa %>% dplyr::select(alloc_key, x, y) %>%write_csv(here("Input_data", "coordinates_SPAM.csv"))


Phys_area_full_cropspread <- read_csv("Rectypes_full_cropspread/Phys_area_full_cropspread.csv")
x=Phys_area_full_cropspread %>% 
  dplyr::rowwise() %>% 
  dplyr::mutate(phys_area_pixl = sum(dplyr::c_across(whea:rest), na.rm = T))

x2=x %>% 
  dplyr::select(alloc_key, x, y, rec_type, tech_type, phys_area_pixl) %>% 
  group_by(alloc_key) %>% 
  mutate(sum_phys_area_pixel = sum(phys_area_pixl))

unique_coord = x2 %>% 
  distinct_at(vars(alloc_key, sum_phys_area_pixel)) %>% 
  left_join(., Phys_area_full_cropspread, by="alloc_key") %>% 
  dplyr::select(alloc_key, x, y, sum_phys_area_pixel) %>% 
  distinct_at(vars(alloc_key, x, y, sum_phys_area_pixel))

write.csv(unique_coord, here("Input_data", "physical_area_per_pixel2.csv"))

















# loading crop usage sheet --------------------------------------------------------------
setwd(here("Input_dat"))
food_cat <- read_csv('foodnonfood.csv')
# Loading crop name sheets
nam_cat <- read_csv('crop_names.csv')
# loading FAO country codes
FAOcntr <- read_csv(here("Input_data", 'FAO_countries.csv'))
FAOcntr <- dplyr::select(FAOcntr, 'ISO3', 'FAOSTAT_CODE', "GAUL_CODE", 'Continent', 'Subcontinent')

# joining all together
names(dat.comb)[8] <- "Crop_short"
names(nam_cat)[2] <- "Crop_short"
names(dat.comb)[2] <- "ISO3"

join.use <- left_join(dat.comb, food_cat, by="Crop_short")
join_FAOcntr <- left_join(join.use, FAOcntr, by="ISO3")
SPAM_dat_tidy <- left_join(join_FAOcntr, nam_cat, by='Crop_short')





AggregateFun_mean <- function(z){ #for yield
  ddply(z, .(name_adm1, crop),
        function(z) mean(z$valu[z$valu!=0]))
}

AggregateFun_sum <- function(g){ #for area
  ddply(g, .(name_adm1, crop),
        function(g) sum(g$valu))
}

CollapseFun <- function(h){
  h %>% dplyr::distinct_at(vars('name_adm1', 'crop'), .keep_all = TRUE)
}

CombineFun <- function(d,s){
  full_join(d,s, by=c("name_adm1", 'crop'))
}



# rename and reoder dataset columns
Clean2Fun <- function(x){
  i <- x %>% dplyr::rename(SPAMcrop_short=Crop_short, 
                           SPAMcrop_long=`SPAM long name`,
                           use=Usage) %>%
    dplyr::select(-c(`No. crt.`, ID))%>%
    arrange(name_adm1, SPAMcrop_short, rec_type, tech_type)
  c <- i[c('ISO3', 'name_cntr', 'name_adm1', 'alloc_key', 'FAOSTAT_CODE', 'GAUL_CODE',   'SPAMcrop_short', 'Continent', 'Subcontinent', 'SPAMcrop_long', 'FAONAMES', 'FAOCODE', 'GROUP', 'use', 'rec_type', 'tech_type', 'unit', 'valu')]
}
SPAM_dat_tidy <-  Clean2Fun(SPAM_dat_tidy)

# Save file - Option long dataset
write.csv(SPAM_dat_tidy, here("Input_dat", "Yld_tidy_all.csv"), row.names=T)

# Option2 - Harvested and Physical area horizontal
SprdFun <- function(x){
  t <- x %>% dplyr::select(-c(unit))  %>%
    spread(rec_type, valu)%>% 
    dplyr::rename(yield_kgha=Y, 
                  phys_area_ha=A,
                  harv_area_ha=H)
  c <- t[c('Continent', 'Subcontinent', 'ISO3', 'name_cntr', 'name_adm1', 'alloc_key', 'FAOSTAT_CODE', 'GAUL_CODE', 'SPAMcrop_short', 'SPAMcrop_long', 'FAONAMES', 'FAOCODE', 'GROUP', 'use', 'tech_type', 'phys_area_ha', 'harv_area_ha', 'yield_kgha')]
} 

sprd_dat <- SprdFun(SPAM_dat_tidy)
write.csv(sprd_dat_fixd, here("Input_dat","SPAMspread_all_fixd.csv"), row.names=T)


