easypackages::packages(c("tidyverse", 'sf', 'raster','sp','dplyr','spatialEco',
                         'exactextractr','tibble', 'here', "readr", "margrittr","tydr",
                         "readxl", "foreign", "data.table", "spData","spDataLarge", "purrr", "rgdal",
                         "ggplot2", "RColorBrewer", "ggplotify", "treemap", "treemapify"))
# Load data
v1= read_csv(here("Input_data", "FAOSTAT_data_9-28-2020_Yields_full.csv")) %>% 
  rename(crop_code = "Item Code")

spm=read_csv(here("Input_data", "SPAM_crops.csv"))

# Getting the FAO CODES of the veggies aggregated in SPAM as "Vegetables"
veg_FAO = c(358:463)
length(veg_FAO) #106 veggies are aggregated in SPAM, of which only 29 are represented here. 
v2=v1 %>% filter(crop_code %in% veg_FAO) %>% write_csv(here("Input_data", "veggies.csv"))

A = read_csv(here("Input_data", "veggies3.csv"))
n1 = read_csv(here("Input_data", "Nutri_cifos.csv"),skip=2) 

# deleting rows
n1.3= filter(n1, !(Family %in% c("Fish", "Meat", "Egg", "Dairy", "Sugar", "Nut", "Oil", "Grain", "Other", "Fruit")))

# Calulatin the percentage for each of the nutrients 
n2.1=n1.3 %>%
  # drop_na() %>% 
  dplyr::select(-c(Weight:Energy)) %>% 
  mutate_at(vars(Protein:N),
            .funs = list(perc = ~ (.)/sum(.)*100)) #validated that all percentage columns add up to 100
# %>% write_csv(here("Input_data", "perc.csv")) 

n2 <- n2.1 %>% dplyr::select(-c(Protein:N)) %>% 
  # gather(n2.1, "content", "value_absolute", "Weight":"N") %>% 
  gather(., "content_perc", "value_perc", "Protein_perc":"N_perc") %>% 
  dplyr::select(-c("Raw NEVO", "Cooked NEVO")) %>%
  rename(usage = X3) %>% 
  right_join(.,v3, by = c("Product"= "CIFOS_nam"))

vec_nutr = pull(n2, var=content_perc) %>% 
  unique() %>% 
  na.omit()

dat_relationship_SNP$Triad_summary_geno

# Creating vectors with subsets per group
macro = vec_nutr[c(4,5,7)]
micro = vec_nutr[c(8:length(vec_nutr))]
fibr  = vec_nutr[6]
water = vec_nutr[2]
weight = vec_nutr[1]
Energy = vec_nutr[3] 

# adding factors
n3=n2 %>% 
  mutate(nutri_type = case_when(
    content_perc %in% macro ~ "Macro_nutrients",
    content_perc %in% micro ~ "Micro_nutrients",
    content_perc %in% weight ~ "Weight",
    content_perc %in% Energy ~ "Energy",
    content_perc %in% water ~ "Water",
    content_perc %in% fibr ~ "Fiber"))

n4 = n3 %>% 
  filter(content_perc != "Water_perc") %>% 
  filter(content_perc != "Weight_perc") %>% 
  filter(content_perc != "NA") %>% 
  filter(content_perc != "Energy_perc") %>% 
  filter(content_perc != "B12_perc") %>% 
  filter(content_perc != "Cholesterol_perc") %>% 
  filter(content_perc != "D_perc") %>% 
  filter(content_perc != "DHA_perc") %>% 
  filter(content_perc != "EPA_perc") %>% 
  filter(nutri_type != "Water") %>% 
  filter(nutri_type != "Weight") %>%
  # filter(content_perc != "NA") %>% 
  filter(nutri_type != "Energy") %>% 
  filter(nutri_type != "Fiber")



# Data
data <- n4 %>% dplyr::select(value_perc, content_perc, veg_type, nutri_type) 


  
a4.4 = an4 %>% group_by(veg_type) %>% 
  drop_na() %>%
  mutate(value_perc = (value_absolute)/sum(value_absolute)*100) #%>% 
  # write_csv(here("Input_data", "check4.csv"))

# Calculates mean, sd, se and IC
n4.5 <- an4 %>%
  group_by(nutri_type, veg_type) %>% #content and nutri_type are dependent
  summarise( 
    n=n(),
    # mean_perc=mean(value_perc),
    # sd_perc=sd(value_perc),
  mean=mean(value_absolute),
sd=sd(value_absolute)) %>%
  # mutate(se=sd/sqrt(n)) %>%
  # mutate(ic=se * qt((1-0.05)/2 + .5, n-1)) %>% 
  # group_by(content, veg_type) %>%
  right_join(., a4.4, by = "veg_type") %>% 
  # dplyr::rename(veg_type = veg_type.x) %>%
    drop_na()
write_csv(n4.5, here("Input_data", "check5.csv"))

# n5 = n4 %>% 
#   dplyr::select(-c("content", value_absolute))

# all values mixed
ggplot(a4.4, aes(x=veg_type, y=value_perc, fill=content))+
  geom_bar(stat="identity", position= "dodge", alpha=0.7)+
  facet_wrap(~nutri_type, scales = "free_y")

# facet
ggplot(n4, aes(x=veg_type, y=value, fill=content))+
  geom_bar(stat="identity", position= "dodge", alpha=0.7) +
  facet_wrap(~nutri_type, scales = "free_y")+
  theme(axis.text.x=element_text(angle = -90, hjust = 0))
  
ggplot(n5, aes(x=veg_type, y=value_perc, fill=content_perc))+
  geom_bar(stat="identity", position= "dodge", alpha=0.7) +
  facet_wrap(~content_perc, scales = "free_y")+
  theme(axis.text.x=element_text(angle = -90, hjust = 0))

# nutrients stacked
ggplot(a4.4, aes(x=veg_type, y=value_perc, fill=content))+
  geom_bar(stat="identity", position= "dodge", alpha=0.7, show.legend = F) +
  # geom_errorbar(data=n4.1, aes(x=veg_type, ymin=mean-sd , ymax=mean+sd), width=0.4, colour="black", alpha=0.9, size=1.3)+
  facet_wrap(~content, scales = "free_y")+
  theme(axis.text.x=element_text(angle = -90, hjust = 0))

# Nutrients averaged
ggplot(n4.5, aes(x=veg_type.x, y=value_absolute, fill=content))+
  # geom_bar(stat="identity", position= "dodge", alpha=0.7, show.legend = F) +
geom_bar(position = "dodge", stat = "summary", fun = "mean", alpha=0.7, show.legend = F)+
  geom_errorbar(aes(x=veg_type.x, ymin=mean-sd , ymax=mean+sd), width=0.4, colour="black", alpha=0.9, size=1.3)+
  facet_wrap(~content, scales = "free_y")+
  theme(axis.text.x=element_text(angle = -90, hjust = 0))


# Perc --------------------------------------------------------------------

# avergae contribution
ggplot(an4, aes(x=veg_type, y=value_absolute, fill=content))+
  # geom_bar(stat="identity", position= "dodge", alpha=0.7, show.legend = F) +
  geom_bar(position = "dodge", stat = "summary", fun = "mean", alpha=0.7, show.legend = F)+
  geom_errorbar(data=n4.5, aes(x=veg_type, ymin=mean-sd , ymax=mean+sd), width=0.4, colour="black", alpha=0.9, size=1.3)+
  facet_wrap(~content, scales = "free_y")+
  theme(axis.text.x=element_text(angle = -90, hjust = 0))



# Analyse by tables -------------------------------------------------------
n4 %>% group_by(nutri_type, value_perc)

write_csv(n4, here("Input_data", "check2.csv"))
                



# absolute --------------------------------------------------------------
# deleting rows
an1.3= filter(n1, !(Family %in% c("Fish", "Meat", "Egg", "Dairy", "Sugar", "Nut", "Oil", "Grain", "Other", "Fruit")))
# 
# # Calulatin the percentage for each of the nutrients 
# an2.1=an1.3 %>%
#   # drop_na() %>% 
#   mutate_at(vars(Weight:N),
#             .funs = list(perc = ~ (.)/sum(.)*100)) #validated that all percentage columns add up to 100
# # %>% write_csv(here("Input_data", "perc.csv")) 

an2 <- an1.3  %>% 
  gather("content", "value_absolute", "Weight":"N") %>%
  # gather(., "content_perc", "value_perc", "Weight_perc":"N_perc") %>% 
  dplyr::select(-c("Raw NEVO", "Cooked NEVO")) %>%
  rename(usage = X3) %>% 
  right_join(.,v3, by = c("Product"= "CIFOS_nam"))

vec_nutra = pull(an2, var=content) %>% 
  unique() %>% 
  na.omit()

# Creating vectors with subsets per group
macro = vec_nutra[c(4,5,7)]
micro = vec_nutra[c(8:length(vec_nutr))]
fibr  = vec_nutra[6]
water = vec_nutra[2]
weight = vec_nutra[1]
Energy = vec_nutra[3] 

# adding factors
an3=an2 %>% 
  mutate(nutri_type = case_when(
    content %in% macro ~ "Macro_nutrients",
    content %in% micro ~ "Micro_nutrients",
    content %in% weight ~ "Weight",
    content %in% Energy ~ "Energy",
    content %in% water ~ "Water",
    content %in% fibr ~ "Fiber"))

an4 = an3 %>% 
  filter(content != "Water") %>% 
  filter(content != "Weight") %>% 
  filter(content != "NA") %>% 
  filter(content != "Energy") %>% 
  filter(content != "B12") %>% 
  filter(content != "Cholesterol") %>% 
  filter(content != "D") %>% 
  filter(content != "DHA") %>% 
  filter(content != "EPA") #%>% 
  # drop_na()


# Data
data <- an4 %>% dplyr::select(value_absolute, content, veg_type, nutri_type) 

# Calculates mean, sd, se and IC
an4.1 <- an4 %>%
  group_by(content, veg_type) %>%
  summarise( 
    n=n(),
    mean=mean(value_absolute),
    sum=sum(value_absolute),
    sd=sd(value_absolute)) %>%
  mutate(se=sd/sqrt(n))  %>%
  mutate(ic=se * qt((1-0.05)/2 + .5, n-1)) %>% 
  # group_by(content, veg_type) %>%
  right_join(., an4, by = "content") %>% 
  dplyr::rename(veg_type = veg_type.x)




# n5 = n4 %>% 
#   dplyr::select(-c("content", value_absolute))

# all values mixed
ggplot(an4.1, aes(x=nutri_type, y=value_absolute, fill=veg_type))+
  geom_bar(stat="identity", position= "dodge", alpha=0.7)

# facet
ggplot(n4, aes(x=veg_type, y=value, fill=content))+
  geom_bar(stat="identity", position= "dodge", alpha=0.7) +
  facet_wrap(~nutri_type, scales = "free_y")+
  theme(axis.text.x=element_text(angle = -90, hjust = 0))

ggplot(n5, aes(x=veg_type, y=value_perc, fill=content_perc))+
  geom_bar(stat="identity", position= "dodge", alpha=0.7) +
  facet_wrap(~content_perc, scales = "free_y")+
  theme(axis.text.x=element_text(angle = -90, hjust = 0))

# nutrients stacked
ggplot(an4, aes(x=veg_type, y=mean, fill=content))+
  geom_bar(stat="identity", position= "stack", alpha=0.7, show.legend = F) +
  geom_errorbar(data=an4.1, aes(x=veg_type, ymin=mean-sd , ymax=mean+sd), width=0.4, colour="black", alpha=0.9, size=1.3)+
  facet_wrap(~content, scales = "free_y")+
  theme(axis.text.x=element_text(angle = -90, hjust = 0))

# Nutrients averaged
ggplot(an4, aes(x=veg_type, y=value_absolute, fill=veg_type))+  
  geom_bar(position = "dodge", stat = "summary", fun = "mean", alpha=0.7, show.legend = F)+
  # geom_errorbar(data=an4.1, aes(x=veg_type, ymin=mean-sd , ymax=mean+sd), width=0.4, colour="black", alpha=0.9, size=1.3)+
  theme(axis.text.x=element_text(angle = -90, hjust = 0))#+
  # facet_wrap(~content, scales = "free_y")
  


# Analyse by tables -------------------------------------------------------
n4 %>% group_by(nutri_type, value_perc)

write_csv(n4, here("Input_data", "check2.csv"))


          
