easypackages::packages(c("tidyverse", 'sf', 'raster','sp','dplyr','spatialEco',
                         'exactextractr','tibble', 'here', "readr",
                         "readxl", "foreign", "data.table", "rbenchmark", "benchmarkme",
                         "validate", "plyr"))

# Set working directory
setwd(here::here("Input_data"))

# Load data ----------------------------------------------------------------
# Yields ------------------------------------------------------------------
dat_TA_yld <- read_csv("spam2010V2r0_global_Y_TA.csv")# Yields - all technologies together, ie complete crop - Data downloaded on 1. of September 2020
dat_TI_yld <- read_csv("spam2010V2r0_global_Y_TI.csv")# Yields - Iirrigated portion of crop
dat_TH_yld <- read_csv("spam2010V2r0_global_Y_TH.csv")# Yields - rainfed high inputs portion of crop
dat_TL_yld <- read_csv("spam2010V2r0_global_Y_TL.csv")# Yields - rainfed low inputs portion of crop
dat_TS_yld <- read_csv("spam2010V2r0_global_Y_TS.csv")# Yields - rainfed subsistence portion of crop
dat_TR_yld <- read_csv("spam2010V2r0_global_Y_TR.csv")# Yields - rainfed portion of crop (= TA - TI, or TH + TL + TS)

# Production
dat_TA_pr <- read_csv("spam2010V2r0_global_P_TA.csv")
dat_TI_pr <- read_csv("spam2010V2r0_global_P_TI.csv")
dat_TH_pr <- read_csv("spam2010V2r0_global_P_TH.csv")
dat_TL_pr <- read_csv("spam2010V2r0_global_P_TL.csv")
dat_TS_pr <- read_csv("spam2010V2r0_global_P_TS.csv")
dat_TR_pr <- read_csv("spam2010V2r0_global_P_TR.csv")

# Physical area--------------------------------
dat_TA_pa <- read_csv("spam2010V2r0_global_A_TA.csv")
dat_TI_pa <- read_csv("spam2010V2r0_global_A_TI.csv")
dat_TH_pa <- read_csv("spam2010V2r0_global_A_TH.csv")
dat_TL_pa <- read_csv("spam2010V2r0_global_A_TL.csv")
dat_TS_pa <- read_csv("spam2010V2r0_global_A_TS.csv")
dat_TR_pa <- read_csv("spam2010V2r0_global_A_TR.csv")

# Harvesting area----------------------------------
dat_TA_ha <- read_csv("spam2010V2r0_global_H_TA.csv")
dat_TI_ha <- read_csv("spam2010V2r0_global_H_TI.csv")
dat_TH_ha <- read_csv("spam2010V2r0_global_H_TH.csv")
dat_TL_ha <- read_csv("spam2010V2r0_global_H_TL.csv")
dat_TS_ha <- read_csv("spam2010V2r0_global_H_TS.csv")
dat_TR_ha <- read_csv("spam2010V2r0_global_H_TR.csv")

# Create a spatial SPAM file 
geo_spam=dplyr::select(dat_TA_pr, x,y, cell5m, name_adm1, iso3) 
# %>% 
# distinct_at(vars(name_adm1))

# Make SPAM spatial 
# projcrs <- "+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0"
# proj_mercator = "+proj=merc +lon_0=0 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs"
proj = "+proj=longlat +datum=WGS84 +no_defs +units=m" # taken from raster file below 
proj_meter = "+datum=WGS84 +units=m +no_defs"
geo_spam1 <- st_as_sf(x = geo_spam,                         
                      coords = c("x", "y"),
                      crs = proj)

# Load administrative unit shapefile which matches the SPAM adm1 zones
Admin0 = st_read(here::here("Input_data","GAUL_GADM_SPAM_ULRIKE", "g2008_0_now","g2008_0.shp"))
Admin1 = st_read(here::here("Input_data","GAUL_GADM_SPAM_ULRIKE", "g2008_1_now","g2008_1.shp")) #this is what we need
Admin2 = st_read(here::here("Input_data","GAUL_GADM_SPAM_ULRIKE", "g2008_2_2020_11_02","g2008_2.shp")) #this is what we need


# Define projection for GAULGADMSPAM
proj = "+proj=longlat +datum=WGS84 +no_defs +units=m" # taken from raster file below 
proj_meter = "+datum=WGS84 +units=m +no_defs"
Admin1 <- st_as_sf(x = Admin1,                         
                   # coords = c("x", "y"),
                   crs = proj)

# Joining SPAM df and GAUL-GADM-SPAM --------------------------------------

# Ulrike's comment: Shape files for GAUL-GADM-SPAM
# Please note that the columns fips0, fips1 and fips2 are the ones which have the codes for admin level 0 (country), level 1 and level 2 which can be linked to SPAM results.
# The matching names are in columns Adm0_name, name1 and name2.# 
# Columns which have names O_fips1, O_fips2, O_name1 and O_name2 refer to an “old” classification system at level 1 and level 2. Its entries differ from the columns without “O_” in some EU countries and a few others where the statistic units did not match the admin units. Please note that the shape files themselves were not modified, only the entries in the dbf tables. 

# adm1 unit match check
geo_spam2 = dat_TA_pr %>% dplyr::select(iso3, cell5m, name_cntr, name_adm1, x, y) %>% 
  distinct_at(vars(name_adm1), .keep_all = T) 

# Define projection for GAULGADMSPAM
proj = "+proj=longlat +datum=WGS84 +no_defs +units=m" # taken from raster file below 
proj_meter = "+datum=WGS84 +units=m +no_defs"
geo_spam3 <- st_as_sf(x = geo_spam2,                         
                   coords = c("x", "y"),
                   crs = proj)


# distinct GGS at NAM1
adm1_distinct=Admin1 %>% as_tibble() %>% 
  distinct_at(vars(NAME1), .keep_all = T) %>%
  write_csv(., here::here("Input_data", "GGS_adm1_dist_full.csv"))

# Converting GGS to dataframe for testing adm1 levels in excel 
GGS_df_adm1 =Admin1 %>% dplyr::select(-geometry) %>% 
  as_tibble() %>% 
  # distinct_at(vars(NAME1), .keep_all = T) %>%
  write_csv(., here::here("Input_data", "GGS_adm1_full.csv"))


# convert to sf (spatial)
proj = "+proj=longlat +datum=WGS84 +no_defs +units=m" # taken from raster file below 
proj_meter = "+datum=WGS84 +units=m +no_defs"
adm1_distinct2 <- st_as_sf(x = adm1_distinct,                         
                   # coords = c("x", "y"),
                   crs = proj)

# Spatial join 
join_SPAM_adm1 = st_join(adm1_distinct2, geo_spam3, join = st_intersects)
# 2857 - 2748 = 109 is the difference between join_SPAM_adm1 and adm1_distinct2

# FUNcheck_incons = function(ped){
#   ped1 = ped  %>% select(id, dam, sire)
#   TestSire_random=ped1$sire==solution_pedsim2$Sire
#   testParents= left_join(ped1, solution_pedsim2) %>%
conS_check = join_SPAM_adm1 %>% 
  rowwise() %>% 
  mutate(testAdm1 = ifelse(NAME1==name_adm1, 1,0))%>%
  mutate(testAdm0 = ifelse(ADM0_NAME==name_cntr, 1,0)) %>% 
  as_tibble() %>% 
  dplyr::select(NAME1, name_adm1, testAdm1, ADM0_NAME, name_cntr, testAdm0) %>%
  write_csv(., here::here("Input_data", "ADm1_0_chck.csv"))

